SystemOrganization addCategory: #'PVpub-AnalogClock'!

PolygonMorph subclass: #SpiralEvent
	instanceVariableNames: 'summary start end'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PVpub-AnalogClock'!

!SpiralEvent commentStamp: 'pv 7/9/2011 19:42' prior: 0!
A SpiralEvent is a PolygonMorph representing a single calendar event. Its shape depends on
1) Spiral
2) start and end time of the event

Instance Variables
	end:			<DateAndTime>
	start:			<DateAndTime>
	summary:		<String>

end
	- end date and time

start
	- start date and time

summary
	- caption to explain what this event is about
	
Responsibilities <Collaborating classes>
	I represent a single calendar event.		
!

!SpiralEvent methodsFor: 'access' stamp: 'pv 5/28/2011 20:48'!
endTime
	^ end ! !

!SpiralEvent methodsFor: 'access' stamp: 'pv 5/28/2011 20:48'!
endTime: endValue
	end := endValue! !

!SpiralEvent methodsFor: 'initialization' stamp: 'pv 11/5/2011 11:14'!
initialize
	super initialize.
	self color: (Color yellow alpha: 0.9).
	summary := TextMorph new.
	summary
		backgroundColor: Color white;
		color: Color black;
		borderWidth: 1;
		borderColor: Color blue;
		borderRaised;
		useRoundedCorners.		
	self addMorph: summary.! !

!SpiralEvent methodsFor: 'access' stamp: 'pv 7/8/2011 21:51'!
pvtLateralShare 
	^ 0.5! !

!SpiralEvent methodsFor: 'initialization' stamp: 'pv 7/8/2011 21:53'!
pvtPlaceSummary
	 | array dx dy vertex1x vertex1y vertex2x vertex2y |
	array := self pvtPlaceSummaryAtIndexPair.
	vertex1x := array first x.
	vertex1y := array first y.
	vertex2x := array second x.
	vertex2y := array second y.
	dx := (vertex2x - vertex1x) * self pvtLateralShare.
	dy := (vertex2y - vertex1y) * self pvtLateralShare.
	^	(vertex1x + dx) rounded @ (vertex1y + dy) rounded! !

!SpiralEvent methodsFor: 'initialization' stamp: 'pv 7/8/2011 21:52'!
pvtPlaceSummaryAtIndexPair
	 | offset midIndex |
	"offset is used to set the summary in the starting corner of the spiral rectangle.
	And to squeeze it to the very beginning when the rectangle gets very small."
	offset := vertices size / 4.
	offset := (offset max: 1) min: 2.
	midIndex := vertices size / 2.
	^ Array with: (vertices at: (midIndex - offset)) with: (vertices at: (midIndex + offset + 1))

! !

!SpiralEvent methodsFor: 'initialization' stamp: 'pv 5/27/2011 20:11'!
setSummaryContents: aString
	self summary contents: aString.! !

!SpiralEvent methodsFor: 'initialization' stamp: 'pv 7/8/2011 21:52'!
setSummaryPosition

	summary center: self pvtPlaceSummary

	"080507pv we could take this value as a starting point and refine in order to
	get a position where the summary-text is as far as possible from other Morphs.
	Take the nearest n neighbours of the summary-text
	Write down the sum of distances from summary-text to those neighbours
	Refine the position of the summary-text in order to maximize the sum of distances."! !

!SpiralEvent methodsFor: 'access' stamp: 'pv 5/28/2011 20:48'!
startTime
	^ start! !

!SpiralEvent methodsFor: 'access' stamp: 'pv 5/28/2011 20:48'!
startTime: startValue
	start := startValue! !

!SpiralEvent methodsFor: 'initialization' stamp: 'pv 5/6/2008 18:19'!
summary
	^ summary! !

PolygonMorph subclass: #WatchHand
	instanceVariableNames: 'myGeometry divisor dateAndTimeDrawn radian radianLag1 myTimeMessage justDropped performanceEvaluator secondsLastDrawTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PVpub-AnalogClock'!

!WatchHand commentStamp: 'pv 7/9/2011 19:44' prior: 0!
Provides a Polygon to show a watchhand on an AnalogClock. This can be a watchhand with arbitrary form for seconds, minutes or hours.
When AnalogClock gets moved or resized, WatchHand will move and resize with it.

WatchHand is a moveable, resizable and steppable polygon-like submorph of AnalogClock.

instance variables:
dateAndTimeDrawn - date and time the watchhand was drawn the last time
ownersRadius		- local representation of owner's radius
ownersCenter		- local representation of owner's center
radian				- orientation of the watchhand in radian
radianLag1			- value of radian in the past, i.e. after executing rotateAt:

Instance Variables
	dateAndTimeDrawn:		<DateAndTime>
	divisor:						<Integer>
	justDropped:				<Boolean>
	myGeometry:				<Dictionary>
	myTimeMessage:			<Symbol>
	performanceEvaluator:		<PerformanceEvaluator>
	radian:						<Float>
	radianLag1:					<Float>
	secondsLastDrawTime:		<DateAndTime>

dateAndTimeDrawn
	- when we send rotateAt: we record the current DateAndTime

divisor
	- in how many parts do we divide a full circle for this time object attribute

justDropped
	- records the fact that a new shape for a watchhand was dropped by the user

myGeometry
	- some geometrical information stored in a dictionary. See initialize

myTimeMessage
	- what to send to a time object in order to get the right attribute

performanceEvaluator
	- someone who can give us information about the jitter

radian
	- current angle in radians

radianLag1
	- last angle in radians. Comparison of radian and radianLag1 used in rotationDue:

secondsLastDrawTime
	- used for performance evaluation (jitter): remember last draw time in seconds
	
Responsibilities <Collaborating classes>
	I represent a visible moving part of the clock
	
	I am able to change my geometry
	<Dictionary>
		
!

WatchHand subclass: #Spiral
	instanceVariableNames: 'nOfVertices spiralCalendar'
	classVariableNames: 'SpiralTrackWidthInPixel VerticesPerFullTurn'
	poolDictionaries: ''
	category: 'PVpub-AnalogClock'!

!Spiral commentStamp: 'pv 7/9/2011 19:40' prior: 0!
Spiral implements a special short term view on a calendar.
For the idea and an implementation in Java see: http://www.emn.fr/x-info/spiraclock/
Permission to use the idea has been requested by the author and granted by Stephane Huot and Pierre Dragicevic on 05/03/2007.

Apart from the usefulness this is nice to look at since it reminds us of mechanical analog clocks which had a spring to store energy.

The spiral is archimedic = distance from the center grows linearly with angle.

Instance Variables
	nOfVertices:		<Integer>
	spiralCalendar:		<SpiralCalendar>

nOfVertices
	- number of vertices needed to represent the spiral

spiralCalendar
	- the calendar from which we can retrieve events

Responsibilities <Collaborating classes>
	I show a spiral-like watchhand with calendar events.
	<SpiralCalendar><SpiralEvent>
	!

!Spiral class methodsFor: 'as yet unclassified' stamp: 'pv 11/14/2011 23:14'!
eventLaysInThePast: minutesToEnd 
	"110731 SwaLint/SmallLint tells us this is a utility method. This
	is true but I want to keep it since it names what we want to
	check here."
	^ minutesToEnd <= 0! !

!Spiral class methodsFor: 'initialize-release' stamp: 'pv 5/29/2011 17:43'!
initialize
	VerticesPerFullTurn := 60.
	SpiralTrackWidthInPixel := 50.! !

!Spiral class methodsFor: 'as yet unclassified' stamp: 'pv 6/20/2011 23:41'!
offsetVectorFromStartPC: startPC
	| pcStartPhi pcStartRNeg |
	pcStartPhi := startPC phi.
	pcStartRNeg := startPC r negated.
	^	((pcStartPhi sin * (pcStartRNeg)) @ 
		(pcStartPhi cos * (pcStartRNeg))).  ! !

!Spiral methodsFor: 'accessing' stamp: 'pv 6/13/2011 16:05'!
addEvent: anEvent
	spiralCalendar addEvent: anEvent.  ! !

!Spiral methodsFor: 'accessing' stamp: 'pv 5/27/2011 22:19'!
eventsWithText: aString
	^ self submorphs select: [:each | each summary text = aString]
! !

!Spiral methodsFor: 'initialization' stamp: 'pv 8/19/2011 18:20'!
initialize
	"self flag: #RQF008."
	super initialize.
      self
		vertices: self vertices
		color: Color blue
		borderWidth: 1
		borderColor: Color blue.
	spiralCalendar := SpiralCalendar new.
	spiralCalendar mySpiralIs: self.
! !

!Spiral methodsFor: 'drawing' stamp: 'pv 11/14/2011 23:19'!
nothingToShowFor: minutesToEnd
	^ (self class eventLaysInThePast: minutesToEnd) | (self pvtNotEnoughVerticesToShowFullEvent: minutesToEnd)! !

!Spiral methodsFor: 'accessing' stamp: 'pv 7/8/2011 21:49'!
pvtDeltaPCFromStartPC: startPC
	| turns |
	turns := 1 max: startPC r // SpiralTrackWidthInPixel.
	nOfVertices := turns * VerticesPerFullTurn.
	^ PolarCoordinates r: startPC r / nOfVertices phi: startPC phi / nOfVertices.	  ! !

!Spiral methodsFor: 'relocating-resizing' stamp: 'pv 7/8/2011 21:48'!
pvtEvent2Rect: anEvent at: dateAndTimeNow 
	| datetimeStart datetimeEnd |
	datetimeStart := anEvent startTime asDateAndTime.
	datetimeEnd := anEvent endTime asDateAndTime.
	^ self 
		pvtRectVerticesStarting: datetimeStart
		ending: datetimeEnd
		at: dateAndTimeNow! !

!Spiral methodsFor: 'drawing' stamp: 'pv 7/8/2011 21:46'!
pvtGetInnerAndOuterLineFrom: startInteger until: endInteger
	"in the next statements, 1 is added since the first vertex is numbered '1'"
	| innerLine outerLine |
	innerLine := self vertices 
		copyFrom: startInteger + 1 + VerticesPerFullTurn  
		to: endInteger + 1 + VerticesPerFullTurn.
	outerLine := (self vertices 
		copyFrom: startInteger + 1
		to: endInteger + 1) reversed.
	^ outerLine , innerLine! !

!Spiral methodsFor: 'drawing' stamp: 'pv 7/31/2011 12:19'!
pvtNotEnoughVerticesToShowFullEvent: minutesToEnd
	^ (minutesToEnd + 1 + VerticesPerFullTurn > nOfVertices)! !

!Spiral methodsFor: 'drawing' stamp: 'pv 11/14/2011 23:20'!
pvtRectVerticesStarting: aDateAndTimeStart ending: aDateAndTimeEnd at: dateAndTimeNow 
	"self flag: #RQF010."
	| minutesToStart minutesToEnd secondsToEnd secondsToStart |
	dateAndTimeDrawn
		ifNil: [^ nil].
	"110731 think in minutes here; we don't care about time units
	smaller than minutes."
	secondsToStart := aDateAndTimeStart asSeconds - dateAndTimeNow asSeconds.
	minutesToStart := secondsToStart + 59 // 60.
	secondsToEnd := aDateAndTimeEnd asSeconds - dateAndTimeNow asSeconds.
	minutesToEnd := secondsToEnd + 59 // 60.
	(self nothingToShowFor: minutesToEnd) ifTrue: [^ nil].
	^ self
		pvtGetInnerAndOuterLineFrom: (minutesToStart max: 0)
		until: minutesToEnd! !

!Spiral methodsFor: 'drawing' stamp: 'pv 7/31/2011 12:28'!
pvtRotateEventsAt: dateAndTimeNow 
	| eventVertices |
	self submorphs
		do: [:each | 
			eventVertices := self pvtEvent2Rect: each at: dateAndTimeNow.
			eventVertices isNil
				ifTrue: [each hide]
				ifFalse: [each setVertices: eventVertices;
						 setSummaryPosition;
						 show]]! !

!Spiral methodsFor: 'accessing' stamp: 'pv 7/8/2011 21:49'!
pvtSetR: r andC: c
	myGeometry 
		at: #ownersCenter put: c;
		at: #ownersRadius put: r.! !

!Spiral methodsFor: 'accessing' stamp: 'pv 7/8/2011 21:50'!
pvtStartPCFromRadius: aFraction
	^ PolarCoordinates r: aFraction phi: ((1 max: aFraction // SpiralTrackWidthInPixel) * 2 * Float pi)  ! !

!Spiral methodsFor: 'drawing' stamp: 'pv 8/19/2011 18:26'!
rotateAt: dateAndTimeNow 
	| |
	"self flag: #RQF009."
	(self owner) ifNil: [^ self].
	(self owner spiraClockOn) ifFalse: [^ self].
	super rotateAt: dateAndTimeNow.
	self pvtRotateEventsAt: dateAndTimeNow.
	radianLag1 := radian.
	dateAndTimeDrawn := dateAndTimeNow	
! !

!Spiral methodsFor: 'relocating-resizing' stamp: 'pv 7/8/2011 21:50'!
setMidnightShapeFromCenter: aPoint andRadius: aFraction
	| p arr deltaPC startPC |
	
	startPC := self pvtStartPCFromRadius: aFraction.
	deltaPC := self pvtDeltaPCFromStartPC: startPC.	
	
	arr := ((1 to: nOfVertices) collect: 
		[:i | 
			"110620pv fehlt hier nicht noch rounded?"
			p := aPoint + (self class offsetVectorFromStartPC: startPC).
			startPC r: (startPC r - deltaPC r).
			startPC phi: (startPC phi - deltaPC phi).
			p]).
	myGeometry at: #MidnightVertices put: arr.! !

!Spiral methodsFor: 'relocating-resizing' stamp: 'pv 8/19/2011 18:20'!
syncMidnightShapeWith: ownerMorph 
	"Intentionally no call to super: different geometry here"
	| r c |
	r := ownerMorph radius.
	c := ownerMorph center.
	(self sameR: r andSameC: c) ifTrue: [^self].
	self 
		setMidnightShapeFromCenter: c 
		andRadius: r;
		makeOpen;
		beSmoothCurve;
		lineWidth: 1;
		show.
	self pvtSetR: r andC: c.
	self rotateAt: dateAndTimeDrawn
	! !

!WatchHand class methodsFor: 'geometry' stamp: 'pv 6/23/2011 00:53'!
getSlopeOf: polygonMorphLongestRadius  

	| pd polygonMorphLongestRadiusB polygonMorphLongestRadiusE |
	polygonMorphLongestRadiusB := polygonMorphLongestRadius beginPoint.	
	polygonMorphLongestRadiusE := polygonMorphLongestRadius endPoint.

	pd := Point 
		x: (polygonMorphLongestRadiusE x - polygonMorphLongestRadiusB x)
		y: (-1.0 * (polygonMorphLongestRadiusE y - polygonMorphLongestRadiusB y)).
	^ 
		(pd x = 0) ifTrue: 
			[(pd y > 0)
				ifTrue: [Float pi / 2] 
				ifFalse: [Float pi * 3 / 2]]
		ifFalse: 
			[(pd x < 0)
				ifTrue: [(pd y / pd x) arcTan + Float pi]
				ifFalse: [(pd y / pd x) arcTan]].! !

!WatchHand methodsFor: 'accessing' stamp: 'pv 5/24/2011 23:31'!
defaultMidnightShape: aBoolean 
	myGeometry at: #defaultMidnightShape put: aBoolean
! !

!WatchHand methodsFor: 'drawing' stamp: 'pv 5/27/2011 13:22'!
drawOn: aCanvas 
	| timeNow |
	"self flag: #RQF003."
	self owner ifNil: [^ self]. 
	self syncMidnightShapeWith: self owner.
	"Performance evaluation:
	Measure accuracy of drawing time compared to nominal time immediately before calling super drawOn: aCanvas"
	(myTimeMessage = #seconds) ifTrue: 
		[timeNow := DateAndTime now.
		performanceEvaluator addDeltaMs:  ((timeNow - secondsLastDrawTime) asMilliSeconds).	
		secondsLastDrawTime := timeNow].
	super drawOn: aCanvas! !

!WatchHand methodsFor: 'initialization' stamp: 'pv 7/8/2011 21:57'!
initialize
	super initialize.
	"initialize everything but default vertices"
	self
		vertices: self vertices
		color: Color red
		borderWidth: 1
		borderColor: Color red.
	radian := 0.
	radianLag1 := nil.
	justDropped := false.
	myGeometry := Dictionary new.
	myGeometry at: #defaultMidnightShape put: true;
		at: #ownersCenter put: 0;
		at: #ownersRadius put: 0.
	self pvtInitializePerformanceEvaluator.! !

!WatchHand methodsFor: 'accessing' stamp: 'pv 6/13/2011 16:29'!
jitterOk
	^ performanceEvaluator percentageOffLimitsOk! !

!WatchHand methodsFor: 'drawing' stamp: 'pv 8/31/2010 14:50'!
justDropped: aBoolean
	justDropped := aBoolean! !

!WatchHand methodsFor: 'relocating-resizing' stamp: 'pv 7/8/2011 21:56'!
pvtAlignWithMidnightShape: aPolygonMorph 
	"A. shift aPolygonMorph in order to have its center coincide with mine.
	 B. rotate aPolygonMorph in order to give it the self orientation as mine."
	| |

	"A."
	self pvtTranslateToMyCenter: aPolygonMorph.
	"B."
	self pvtRotateToMyOrientation: aPolygonMorph.
! !

!WatchHand methodsFor: 'private' stamp: 'pv 7/8/2011 21:53'!
pvtInRadian: aTime 
	"080413 should be a class method."
	| timeValue delta |
	delta := 0.0.
	timeValue := (aTime perform: myTimeMessage) asFloat.
	(myTimeMessage = #hours) ifTrue: 
		[delta := aTime minutes / 60.0].
	^ 2 * Float pi * (timeValue + delta) / divisor! !

!WatchHand methodsFor: 'initialization' stamp: 'pv 7/8/2011 21:56'!
pvtInitializePerformanceEvaluator
	secondsLastDrawTime := DateAndTime now.
	performanceEvaluator := PerformanceEvaluator new.
	performanceEvaluator 
		leftLimit: 0;
		rightLimit: 2000;
		percentageOffLimitsThreshold: 0.
! !

!WatchHand methodsFor: 'drawing' stamp: 'pv 7/8/2011 21:57'!
pvtRotateAboutReferencePosition: aPolygonMorph by: aRadian 
	| ownersCenter |
	ownersCenter := myGeometry at: #ownersCenter.
	self assert: (aPolygonMorph referencePosition = ownersCenter).
	aPolygonMorph setVertices: (aPolygonMorph vertices collect: 
		[ :vertex | 
			vertex 
				rotateBy: aRadian 
				about: ownersCenter])! !

!WatchHand methodsFor: 'drawing' stamp: 'pv 7/8/2011 21:58'!
pvtRotateToMidnight: aPolygonMorph from: polygonMorphLongestRadius

	| polygonMorphSlope |
	
	polygonMorphSlope := self class getSlopeOf: polygonMorphLongestRadius.
	
	"Float pi / 2 is the midnight position"
	self 
		pvtRotateAboutReferencePosition: aPolygonMorph
		by: (Float pi / 2) - polygonMorphSlope.
! !

!WatchHand methodsFor: 'relocating-resizing' stamp: 'pv 7/8/2011 21:58'!
pvtRotateToMyOrientation: aPolygonMorph
	| myLongestRadius polygonMorphLongestRadius denominator nominator |
	myLongestRadius := self 
		findLongestRadiusFrom: (myGeometry at: #ownersCenter).	
	polygonMorphLongestRadius := aPolygonMorph 
		findLongestRadiusFrom: aPolygonMorph referencePosition.
	(myLongestRadius isNil | polygonMorphLongestRadius isNil) ifTrue: [^ self].
	
	self 
		pvtRotateToMidnight: aPolygonMorph 
		from: polygonMorphLongestRadius. 
	"Transcript show: 'after rotate:from: old hand=', self vertices asString, 'new hand=', aPolygonMorph vertices asString."
	"Transcript show: 'after rotate:from: my length=', (myLongestRadius endPoint dist: myLongestRadius beginPoint) asString, 'polygon length=', (polygonMorphLongestRadius endPoint dist: polygonMorphLongestRadius beginPoint) asString."

	nominator := (myLongestRadius endPoint dist: myLongestRadius beginPoint) asFloat.
	denominator := (polygonMorphLongestRadius endPoint dist: polygonMorphLongestRadius beginPoint) asFloat.
	self 
		pvtScaleToMySize: aPolygonMorph
		byFactor: nominator / denominator.
	"Transcript show: 'after scaleToMySize:byFactor: new hand=', aPolygonMorph vertices asString"		  ! !

!WatchHand methodsFor: 'relocating-resizing' stamp: 'pv 7/8/2011 21:55'!
pvtScaleToMySize: aPolygonMorph byFactor: aFloat
	"080522 consider merge with scaleVertices:by:aroundCenter:"
	| ownersCenter |
	ownersCenter := myGeometry at: #ownersCenter.
	self assert: (aPolygonMorph referencePosition = ownersCenter).
	"Transcript show: 'scaleToMySize:byFactor: Factor=', aFloat asString."
	aPolygonMorph setVertices: (aPolygonMorph vertices collect: 
		[:vertex | 
			(ownersCenter x + ((vertex x - ownersCenter x) * aFloat)) @ (ownersCenter y + ((vertex y - ownersCenter y) * aFloat)) rounded]).

! !

!WatchHand methodsFor: 'relocating-resizing' stamp: 'pv 7/8/2011 21:55'!
pvtSetDefaultMidnightShapeFromCenter: aPoint andRadius: aFloat 
	| lozengeLengthFromCenter lozengeWidthFromCenter |
	lozengeLengthFromCenter := aFloat * (myGeometry at: #Size) max: 2.
	lozengeWidthFromCenter := lozengeLengthFromCenter * (1.0 - (myGeometry at: #Size)) / 12 max: 1.
	myGeometry at: #MidnightVertices put: (Array 
		with: aPoint + (0 @ lozengeWidthFromCenter)
		with: aPoint + ((-1 * lozengeWidthFromCenter) @ 0)
		with: aPoint + (0 @ -1 * lozengeLengthFromCenter)
		with: aPoint + (lozengeWidthFromCenter @ 0))! !

!WatchHand methodsFor: 'relocating-resizing' stamp: 'pv 7/8/2011 21:56'!
pvtTranslateToMyCenter: aPolygonMorph
	"080522 consider merge with translateVertices:fromA:toB:"
	| translation |
	translation := aPolygonMorph referencePosition - (myGeometry at: #ownersCenter).
	aPolygonMorph setVertices: 	
		(aPolygonMorph vertices collect: 
			[:vertex | vertex - translation]).
	aPolygonMorph setRotationCenterFrom: (myGeometry at: #ownersCenter).
	
! !

!WatchHand methodsFor: 'accessing' stamp: 'pv 6/13/2011 16:29'!
radian	
	^ radian! !

!WatchHand methodsFor: 'drawing' stamp: 'pv 6/12/2011 23:47'!
rotateAt: dateAndTimeNow 
	"080406 rotation was due; so now update polygon vertices."
	| ownersCenter |
	ownersCenter := myGeometry at: #ownersCenter.
	self setVertices: ((myGeometry at: #MidnightVertices) 
		collect: [:vertex | (vertex 
			rotateBy: -1 * radian 
			about: ownersCenter) rounded]).
	radianLag1 := radian.	
	dateAndTimeDrawn := dateAndTimeNow.
! !

!WatchHand methodsFor: 'drawing' stamp: 'pv 6/12/2011 23:46'!
rotateLazyAt: dateAndTimeNow 
	"080406 rotation was due; so now update polygon vertices."
	| |
	(self rotationDue: dateAndTimeNow) ifFalse: [^ self].
	self rotateAt: dateAndTimeNow.
! !

!WatchHand methodsFor: 'drawing' stamp: 'pv 7/31/2011 00:47'!
rotationDue: dateAndTimeNow 
	radian := self pvtInRadian: dateAndTimeNow asTime.
	justDropped ifTrue: [justDropped := false. ^ true].
	"second operand of or-operation is specific to the spiral and a time increment of 1 hour: 
	in this specific case angle for the spiral is the same but spiral events need to be drawn"
	"110731 therefore you should implement rotationDue: in Spiral."
	^ 	(radian ~= radianLag1) | 
		((radian = radianLag1) & (dateAndTimeDrawn isNil not) and: [dateAndTimeNow hour ~= dateAndTimeDrawn hour])! !

!WatchHand methodsFor: 'accessing' stamp: 'pv 6/23/2011 00:56'!
sameR: r andSameC:  c 
	^ ((myGeometry at: #ownersRadius) = r) & ((myGeometry at: #ownersCenter) = c)! !

!WatchHand methodsFor: 'relocating-resizing' stamp: 'pv 6/2/2011 17:41'!
setMidnightShapeFromCenter: aPoint andRadius: aFloat
	| tmp |
	tmp := PolygonMorph new. 
	tmp setVertices: (myGeometry at: #MidnightVertices).
	myGeometry at: #MidnightVertices put: (tmp 
		translateFromA: (myGeometry at: #ownersCenter) toB: aPoint) vertices.
	myGeometry at: #MidnightVertices put: (tmp 
		scaleByFactor: (aFloat / (myGeometry at: #ownersRadius)) aroundCenter: aPoint) vertices

! !

!WatchHand methodsFor: 'private' stamp: 'pv 7/8/2011 21:54'!
setMidnightVerticesFrom: aUserPolygonMorph 
	"User has dropped aPolygonMorph on AnalogClock
	and wants us to use the shape of aPolygonMorph for self."
	
	"Check if it will be possible to do the operation. If not: keep the current watchhand."
	| |

	(aUserPolygonMorph isNil | aUserPolygonMorph referencePosition isNil) ifTrue: 
		[^ self].
	
	self pvtAlignWithMidnightShape: aUserPolygonMorph.
	myGeometry at: #MidnightVertices put: (aUserPolygonMorph vertices)! !

!WatchHand methodsFor: 'accessing' stamp: 'pv 5/27/2011 16:42'!
setTimeMessage: constString divisor: anInteger andSize: aFloat 
	myTimeMessage := constString. "what to send to a time object in order to get the right attribute"
	divisor := anInteger. "in how many parts do we divide a full circle for this time object attribute"
	myGeometry at: #Size put: aFloat "a number between 0.0 and 1.0 which shows the size of self w.r.t. the clock radius."! !

!WatchHand methodsFor: 'relocating-resizing' stamp: 'pv 9/18/2011 22:27'!
syncMidnightShapeWith: aMorph 
	| r c |
	"self
		flag: #RQF002;
		flag: #RQF007."
	r := aMorph radius.
	c := aMorph center.
	"Transcript show: ' r=', r, ' c=', c."
	(self sameR: r andSameC: c) ifTrue: [ ^ self ].
	"this is the default way to set myMidnightVertices, As soon as a new polygon shape
	was dropped on AnalogClock, this will replace myMidnightVertices. Undo ?"
	(myGeometry at: #defaultMidnightShape) 
		ifTrue: [self pvtSetDefaultMidnightShapeFromCenter: c andRadius: r]
		ifFalse: [self setMidnightShapeFromCenter: c andRadius: r].
	myGeometry 
		at: #ownersCenter put: c;
		at: #ownersRadius put: r.
	self rotateAt: dateAndTimeDrawn! !

!WatchHand methodsFor: 'halos and balloon help' stamp: 'pv 8/31/2010 18:18'!
wantsHaloFromClick
	^false! !

EllipseMorph subclass: #Dial
	instanceVariableNames: 'figureMorphs tickmarkMorphs myGeometry'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PVpub-AnalogClock'!

!Dial commentStamp: 'pv 11/10/2011 18:54' prior: 0!
A Dial is xxxxxxxxx.

Instance Variables
	figureMorphs:		<FigureMorph>
	myGeometry:		<Dictionary>
	textStyle:			<TextStyle>
	tickmarkMorphs:	<TickmarkMorph>

figureMorphs
	- shows figures from 1 to 12

myGeometry
	- container for some geometric properties

textStyle
	- text style (e.g. centered)

tickmarkMorphs
	- show tickmarks
!

!Dial class methodsFor: 'as yet unclassified' stamp: 'pv 8/18/2011 14:02'!
fractionOfTwoPiFrom: aNominator by: aDenominator
	^ Float pi * 2 asFloat * (Fraction numerator: aNominator denominator: aDenominator) asFloat! !

!Dial methodsFor: 'drawing' stamp: 'pv 9/18/2011 22:47'!
drawOn: aCanvas 
	| |
	"self flag: #RQF003."
	self owner ifNil: [^ self]. 
	self syncMidnightShapeWith: self owner.
	super drawOn: aCanvas! !

!Dial methodsFor: 'drawing' stamp: 'pv 10/31/2011 23:46'!
hideFigure3 
	(figureMorphs at: 3) hide! !

!Dial methodsFor: 'drawing' stamp: 'pv 8/20/2011 06:59'!
hideTickmarks
	tickmarkMorphs do: 
		[:each | each hide]! !

!Dial methodsFor: 'initialization' stamp: 'pv 10/31/2011 23:03'!
initialize
	super initialize.
	self
		color: (Color yellow alpha: 0.5);
		borderColor: Color blue.
! !

!Dial methodsFor: 'initialization' stamp: 'pv 11/1/2011 23:33'!
initializeSubmorphs
	self 
		pvtInitializeFigureMorphs; 
		pvtInitializeTickmarkMorphs; 
		"disable editing on me or my submorphs"
		lock: true.
	myGeometry := Dictionary new.
	myGeometry at: #defaultMidnightShape put: true;
		at: #ownersCenter put: 0;
		at: #ownersRadius put: 0.! !

!Dial methodsFor: 'initialization' stamp: 'pv 1/20/2013 08:18'!
pvtInitializeFigureMorphs
	| figureMorph |
	figureMorphs := (1 to: 12)
				collect: [:each | 
					figureMorph := Figure new.
					self addMorph: figureMorph.
					figureMorph newContents: each asString.
						 "font: font;
						 setTextStyle: textStyle"]! !

!Dial methodsFor: 'initialization' stamp: 'pv 1/20/2013 08:19'!
pvtInitializeTickmarkMorphs
	| tickmarkMorph |
	tickmarkMorphs := (1 to: 60) collect: 
		[:each | 
			tickmarkMorph := Tickmark new.
			self addMorph: tickmarkMorph.
			"110529 last statement shall evaluate to a component of our new array"
			tickmarkMorph]! !

!Dial methodsFor: 'geometry' stamp: 'pv 8/18/2011 14:01'!
pvtOffsetVectorFromRadius: aFloat withRelativeLength: relLength forIndex:  index ofTotal: total
	"080413pv without rounded this would yield float-bounds which in turn will lead to pixel errors."
	| f x y |
	f :=	self class fractionOfTwoPiFrom: index by: total.   
	x := aFloat * relLength * f sin.
	y := 0 - (aFloat * relLength * f cos).
	^ x rounded @ y rounded! !

!Dial methodsFor: 'geometry' stamp: 'pv 11/14/2011 23:12'!
pvtSetFiguresFromCenter: aPoint andRadius: aFloat 
	| indexInFigureMorphs vector textStyle |
	textStyle := TextStyle default copy.
	textStyle alignment: 2.
	"centered"
	figureMorphs
		do: [:each | 
			indexInFigureMorphs := figureMorphs indexOf: each.
			vector := self
						pvtOffsetVectorFromRadius: aFloat
						withRelativeLength: 0.85
						forIndex: indexInFigureMorphs
						ofTotal: figureMorphs size.
			each
				fontName: 'BitstreamVeraSans'
				size: (aFloat // 8 min: 36);
				setTextStyle: textStyle;
				center: aPoint + vector]! !

!Dial methodsFor: 'geometry' stamp: 'pv 8/18/2011 13:50'!
pvtSetTickmarksFromCenter: aPoint andRadius: aFloat
	| tickmarkRadiusBig tickmarkRadiusSmall indexInTickmarkMorphs vector |
	tickmarkRadiusBig := ((aFloat / 70) max: 1).
	tickmarkRadiusSmall := ((aFloat / 130) max: 1).
	tickmarkMorphs do: 
		[:each |
		indexInTickmarkMorphs := tickmarkMorphs indexOf: each. 
		vector := self pvtOffsetVectorFromRadius: aFloat withRelativeLength: 0.95 forIndex: indexInTickmarkMorphs ofTotal: tickmarkMorphs size.
		each
			center: aPoint + vector;
			extent: ((indexInTickmarkMorphs  \\ 5 = 0) 
				ifTrue: [tickmarkRadiusBig @ tickmarkRadiusBig]
				ifFalse: [tickmarkRadiusSmall @ tickmarkRadiusSmall])].
! !

!Dial methodsFor: 'drawing' stamp: 'pv 8/16/2011 23:08'!
rotateLazyAt: dateAndTime
	"my steppable submorph dateMorph is concerned only"
	self submorphsDo: 
		[ :each | each rotateLazyAt: dateAndTime ].  ! !

!Dial methodsFor: 'geometry' stamp: 'pv 8/18/2011 13:58'!
sameR: r andSameC:  c 
	^ ((myGeometry at: #ownersRadius) = r) & ((myGeometry at: #ownersCenter) = c)! !

!Dial methodsFor: 'geometry' stamp: 'pv 10/31/2011 23:10'!
setMidnightShapeFromCenter: aPoint andRadius: aFloat
	| radiusAsInteger |
	radiusAsInteger := aFloat rounded.
	self bounds: (aPoint - (radiusAsInteger @ radiusAsInteger) corner: aPoint + (radiusAsInteger @ radiusAsInteger)).
	self pvtSetFiguresFromCenter: aPoint andRadius: aFloat.
	self pvtSetTickmarksFromCenter: aPoint andRadius: aFloat.

! !

!Dial methodsFor: 'drawing' stamp: 'pv 10/31/2011 23:46'!
showFigure3 
	(figureMorphs at: 3) show! !

!Dial methodsFor: 'drawing' stamp: 'pv 8/20/2011 07:00'!
showTickmarks
	tickmarkMorphs do: 
		[:each | each show].
	! !

!Dial methodsFor: 'geometry' stamp: 'pv 9/18/2011 22:48'!
syncMidnightShapeWith: aMorph 
	| r c |
	r := aMorph radius.
	c := aMorph center.
	"Transcript show: ' r=', r, ' c=', c."
	(self sameR: r andSameC: c) ifTrue: [ ^ self ].
	self setMidnightShapeFromCenter: c andRadius: r.
	myGeometry 
		at: #ownersCenter put: c;
		at: #ownersRadius put: r.
! !

TextMorph subclass: #DateText
	instanceVariableNames: 'stringLag1 string myTimeMessage myGeometry'
	classVariableNames: 'BlankAndYearSize'
	poolDictionaries: ''
	category: 'PVpub-AnalogClock'!

!DateText class methodsFor: 'class initialization' stamp: 'pv 11/15/2011 23:41'!
initialize 
	BlankAndYearSize := 5! !

!DateText methodsFor: 'drawing' stamp: 'pv 11/13/2011 13:07'!
drawOn: aCanvas 
	self owner
		ifNil: [^ self].
	(self syncMidnightShapeWith: self owner) 
		ifTrue: [super drawOn: aCanvas].! !

!DateText methodsFor: 'geometry' stamp: 'pv 11/13/2011 13:07'!
horizontalExtentAsFloat
	^ self extent x asFloat! !

!DateText methodsFor: 'initialization' stamp: 'pv 11/13/2011 13:07'!
initialize
	super initialize.
	self lock: true.
	stringLag1 := ''.
	myGeometry := Dictionary new.
	myGeometry at: #defaultMidnightShape put: true;
		at: #ownersCenter put: 0;
		at: #ownersRadius put: 0.	! !

!DateText methodsFor: 'geometry' stamp: 'pv 11/13/2011 15:15'!
pvtSetDateFromCenter: aPoint andRadius: aFloat
	self center: aPoint + ((aFloat * 0.5) rounded @ 0);
		font: (StrikeFont familyName: 'BitstreamVeraSans' size: ((aFloat // 8) min: 24));
		right: aPoint x + (aFloat * 0.9) rounded.
	"Transcript show: (aPoint x + (aFloat * 0.9) rounded) asString, ' '"! !

!DateText methodsFor: 'drawing' stamp: 'pv 11/16/2011 00:18'!
rotateAt: dateAndTimeNow 
	"select day and month and discard year"
	string lastSpacePosition > -1
		ifTrue: [string truncateTo: string lastSpacePosition].
	self
		contents: (string truncateTo: string size - BlankAndYearSize).
	stringLag1 := string! !

!DateText methodsFor: 'drawing' stamp: 'pv 11/13/2011 13:08'!
rotateLazyAt: dateAndTimeNow 
	(self rotationDue: dateAndTimeNow) ifFalse: [^ self].
	self rotateAt: dateAndTimeNow.! !

!DateText methodsFor: 'drawing' stamp: 'pv 11/13/2011 13:08'!
rotationDue: now
	string := ' ', (now perform: myTimeMessage) asString, ' '.
	^ (string sameAs: stringLag1) not
! !

!DateText methodsFor: 'geometry' stamp: 'pv 11/16/2011 00:01'!
sameR: r andSameC:  c 
	| unchanged |
	unchanged := ((myGeometry at: #ownersRadius) = r) & ((myGeometry at: #ownersCenter) = c).
	unchanged ifFalse: [
	myGeometry 
		at: #ownersCenter put: c;
		at: #ownersRadius put: r.].
	^ unchanged! !

!DateText methodsFor: 'geometry' stamp: 'pv 11/13/2011 13:09'!
setMidnightShapeFromCenter: aPoint andRadius:  aFloat
	((self horizontalExtentAsFloat) > aFloat) 
		ifTrue: 
			[^ false] 
		ifFalse:	
			[self pvtSetDateFromCenter: aPoint andRadius: aFloat.
			^ true]! !

!DateText methodsFor: 'initialization' stamp: 'pv 11/13/2011 13:09'!
setTimeMessage: timeSelector divisor: d andSize: size 
	myTimeMessage := timeSelector "what to send to a time object in order to get the right attribute"
	"divisor := nil. - does not make sense here" 
	"in how many parts do we divide a full circle for this time object attribute"
	"mySize := size - does not make sense here" "which share of a radius do you need"! !

!DateText methodsFor: 'geometry' stamp: 'pv 11/16/2011 00:12'!
syncMidnightShapeWith: aMorph 
	"Scale self according to its owner. In addition:
	If owner is too small setMidnightShapeFromCenter:andRadius: will return false and will disable drawing.
	Otherwise it will return true in order to draw.
	When we are disabled it is our owner who will enable us
	again."
	| r c |
	r := aMorph radius.
	c := aMorph center.
	"Transcript show: ' r=', r, ' c=', c."
	(self sameR: r andSameC: c)
		ifTrue: [^ true].
	(self setMidnightShapeFromCenter: c andRadius: r)
		ifTrue: [self show.
			^ true]
		ifFalse: [self hide.
			^ false]! !

!DateText methodsFor: 'halos and balloon help' stamp: 'pv 11/13/2011 13:10'!
wantsHaloFromClick
	^false! !

TextMorph subclass: #Figure
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PVpub-AnalogClock'!

!Figure methodsFor: 'drawing' stamp: 'pv 8/13/2011 23:40'!
rotateLazyAt: dateAndTime 
	^ self! !

Morph subclass: #AnalogClock
	instanceVariableNames: 'configPanel viewController'
	classVariableNames: 'FullWorld'
	poolDictionaries: ''
	category: 'PVpub-AnalogClock'!

!AnalogClock commentStamp: 'pv 11/10/2011 17:30' prior: 0!
An AnalogClock is the class which is used to start the application: 
AnalogClock new openInWorld. 
It embeds different graphical elements which represent visible parts of the clock. It holds an organizer which takes care of the "mechanics" of the clock.

Instance Variables
	configPanel:		<ConfigPanel>
	viewController 		<ViewController>

configPanel
	- used as a graphical user interface for testing purposes

viewController
	- combines presentation (view) and control of user interaction (controller)
	
MorphExtension (see pvtInitializeConfiguration)
	configPanelOn 		visibility of config panel	
	spiraClockOn 		visibility of spiral watchhand
	tickmarksOn 		visibility of tickmarks
	dateOn 			visibility of date information
	dateAndTimeSource do we take date and time from operating system? do we add an offset?		
	running 			is clock stopped or does it run?
				
Responsibilities <Collaborating classes>
	I  launch the application (via Morphic or standalone)
	
	I provide an extension to the halo menu 
	<HandMorph>
	
	I accept two types of morphs to be dropped on me (see acceptDroppingMorph:event:) 
	<viewController><WatchHand>
	
	
	
	
	
	

	
	
	
	
	
	
	
!

!AnalogClock class methodsFor: 'metrics' stamp: 'pv 6/2/2011 16:04'!
Loc
	| instanceLoc classLoc |
	instanceLoc := classLoc := 0.
	(PackageInfo named: 'PVpub-AnalogClock') classes do:
		[:class |
			instanceLoc := class linesOfCode + instanceLoc.
			classLoc := class class linesOfCode + classLoc].
	Transcript show: 'instanceLoc=', instanceLoc asString, ', classLoc=', classLoc asString ! !

!AnalogClock class methodsFor: 'standAlone' stamp: 'pv 6/11/2008 22:44'!
addApplication
    "AnalogClock addApplication"
    Smalltalk addToStartUpList: AnalogClock.! !

!AnalogClock class methodsFor: 'standAlone' stamp: 'pv 6/11/2008 22:44'!
clearApplication
    "AnalogClock clearApplication"
    Smalltalk removeFromStartUpList: AnalogClock.! !

!AnalogClock class methodsFor: 'standAlone' stamp: 'pv 6/11/2008 22:43'!
closeAllOpenWindowsMessage
    ^'Do you really want to close all windows?' translated! !

!AnalogClock class methodsFor: 'standAlone' stamp: 'pv 6/11/2008 22:42'!
closeAllWindows
    "AnalogClock closeAllWindows"
    (SystemWindow windowsIn: World satisfying: [:w | w model canDiscardEdits])
        do: [:w | w delete]! !

!AnalogClock class methodsFor: 'standAlone' stamp: 'pv 6/15/2011 20:58'!
disableFeatures
     	Preferences disable: #warnIfNoSourcesFile.
     	Preferences disable: #warnIfNoChangesFile.
     	Preferences disable: #cmdDotEnabled.
 	Preferences enable: #eToyFriendly.
	Preferences enable: #noviceMode.
     	Preferences compileHardCodedPref: #cmdGesturesEnabled enable: false.
     	Preferences compileHardCodedPref: #cmdKeysInText enable: false.
	"Steve Wessels: never execute this except for when you want to make a deployed image"
	HaloMorph compileSilently: 'addCircleHandles' classified: 'disable'! !

!AnalogClock class methodsFor: 'standAlone' stamp: 'pv 6/15/2008 22:19'!
fullWorld
	FullWorld isNil ifTrue: 
		[self fullWorld: false].
	^ FullWorld! !

!AnalogClock class methodsFor: 'standAlone' stamp: 'pv 5/28/2008 21:53'!
fullWorld: aBoolean
	"AnalogClock fullWorld: true"
	FullWorld := aBoolean! !

!AnalogClock class methodsFor: 'standAlone' stamp: 'pv 5/28/2008 22:15'!
launchFullWorld
    "LaserGame launchFullWorld"
    | morph |
    self fullWorld: true.
    morph := self new.
    morph openInWorld.
    ^morph ! !

!AnalogClock class methodsFor: 'standAlone' stamp: 'pv 5/28/2008 22:14'!
launchStandard
	"AnalogClock launchStandard"
	| morph |
    self fullWorld: false.
    morph := self new.
    morph openInHand.
    ^ morph
	! !

!AnalogClock class methodsFor: 'standAlone' stamp: 'pv 1/1/2009 22:14'!
prepareApplication
	"AnalogClock prepareApplication"
	(self confirm: self prepareApplicationWarningMessage) ifFalse: [^ self].
	(SelectionMenu confirm: self closeAllOpenWindowsMessage) ifFalse: [^ self].
	self closeAllWindows.
	self launchFullWorld.
	self disableFeatures.
	self addApplication.
	SmalltalkImage current saveAs! !

!AnalogClock class methodsFor: 'standAlone' stamp: 'pv 6/11/2008 22:48'!
prepareApplicationWarningMessage
    ^'Really make a stand-alone image?
    Any existing AnalogClock morphs should be closed.
	A new one will be re-created by this process.
	Important developer tools will be disabled.
	The image should be saved a new name as the final step of this process.' translated ! !

!AnalogClock class methodsFor: 'standAlone' stamp: 'pv 6/11/2008 22:46'!
purgeAndAnswerSoleInstance
    "AnalogClock purgeAndAnswerSoleInstance"
    | coll result |
    coll := self allInstances.
    coll size > 1 ifTrue: [2 to: coll size do: [:index |
            | morph |
            morph := coll at: index.
            morph delete]].
    result := coll isEmpty
        ifTrue: [nil]
        ifFalse: [coll first].
    ^result! !

!AnalogClock class methodsFor: 'initialize-release' stamp: 'pv 6/11/2008 22:47'!
startUp
    | soleInstance |
    soleInstance := self purgeAndAnswerSoleInstance.
    soleInstance isNil ifTrue: [soleInstance := self launchFullWorld].
    "080611pv adapt:
	GridFactory reSeed.
    soleInstance newGame."! !

!AnalogClock methodsFor: 'layout' stamp: 'pv 8/20/2011 06:49'!
acceptDroppingMorph: aMorph event: evt 
	| watchhand |
	<lint: 'Sends "questionable" message' rationale: 'We need to know if the dropped morph responds to this message at least' author: 'Pascal Vollmer'>
	"self flag: #RQF012; flag: #RQF015."
	"2011-05-24pv code unchanged although SwaLint-warning:
	isKindOf "
	"TextMorph"
	(aMorph respondsTo: #contents)
		ifTrue: ["self flag: #RQF012; flag: #RQF015."
			"2011-05-24pv code unchanged although
			SwaLint-warning: isKindOf"
			"TextMorph"
			(viewController watchhands at: #Spiral)
				addEvent: aMorph text string.
			"self flag: #RQF012; flag: #RQF015."
			"2011-05-24pv code unchanged although
			SwaLint-warning: isKindOf"
			"TextMorph"
			^ self].
	(aMorph respondsTo: #vertices)
		ifTrue: ["PolygonMorph"
			watchhand := self pvtClosestWatchhandTo: aMorph.
			"PolygonMorph"
			watchhand setMidnightVerticesFrom: aMorph;
				 defaultMidnightShape: false].
	aMorph dismissMorph
	"Intentionally no call to super: aMorph has been deleted"
	"super acceptDroppingMorph: aMorph event: evt."! !

!AnalogClock methodsFor: 'menus' stamp: 'pv 7/7/2011 23:37'!
addCustomMenuItems: aCustomMenu hand: aHandMorph 
	"self flag: #RQF015."
	super addCustomMenuItems: aCustomMenu hand: aHandMorph.
	self pvtAddButtonsFor: aCustomMenu.! !

!AnalogClock methodsFor: 'accessing' stamp: 'pv 8/20/2011 11:03'!
clockIF
	^ viewController if! !

!AnalogClock methodsFor: 'meta-actions' stamp: 'pv 11/10/2011 18:57'!
dismissMorph 
	"When AnalogClock is deleted, ConfigPanel needs to get deleted as well."
	viewController configPanel ifNotNil: [viewController configPanel delete].
	super dismissMorph ! !

!AnalogClock methodsFor: 'initialization' stamp: 'pv 11/13/2011 14:03'!
initialize
	"self flag: #RQF001; flag: #RQF018."
	super initialize.
	self bounds: (100 @ 100 corner: 400 @ 400);
		color: Color transparent.
	self
		 pvtInitializeAnalogClockViewController;
		 pvtSetAbilityToMove.
	"Drop behavior: accept TextMorph-s with ICal-Events, accept
	PolygonMorph-s with replacements for watchhands"
	self dropEnabled: true

! !

!AnalogClock methodsFor: 'configuring' stamp: 'pv 11/10/2011 21:34'!
pvtAddButtonsFor: aCustomMenu
	aCustomMenu 
		add: ((viewController getProperty: #spiraClockOn) 	ifTrue: ['switch off Spiraclock']	ifFalse: ['switch on Spiraclock']) translated
		target: viewController 
		action: #toggleSpiraClock;
		add: ((viewController getProperty: #configPanelOn) ifTrue: ['hide ConfigPanel'] ifFalse: ['show ConfigPanel']) translated
		target: viewController 
		action: #toggleConfigPanel;
		add: ((viewController getProperty: #tickmarksOn) ifTrue: ['hide tickmarks'] ifFalse: ['show tickmarks']) translated
		target: viewController 
		action: #toggleTickmarks;
		add: ((viewController getProperty: #dateOn) ifTrue: ['hide date'] ifFalse: ['show date']) translated
		target: viewController
		action: #toggleDate.  ! !

!AnalogClock methodsFor: 'dropping/grabbing' stamp: 'pv 11/13/2011 15:04'!
pvtClosestWatchhandTo: aMorph 
	"080522 Proposal for an algorithm:
	choose a line which goes through referencePosition of the
	polygon and which is best regression line.
	Then take sum of distances from endpoints of this
	referenceline of the new watchhand to endpoints of
	referenceline of the different existing watchhands. Take the
	morph with the smallest distance.
	Here we just use the seconds watchhand as a shortcut."
	^ viewController watchhandSec! !

!AnalogClock methodsFor: 'initialization' stamp: 'pv 9/18/2011 22:40'!
pvtInitializeAnalogClockViewController
	viewController := AnalogClockViewController new.
	viewController bounds: self bounds.
	self addMorph: viewController! !

!AnalogClock methodsFor: 'geometry' stamp: 'pv 7/7/2011 23:46'!
pvtPlaceConfigPanelWhichIs: aRectangleMorph
	| southEastVector northEastVector analogClockCorners candidatePositions |
	"default pos: lower right edge of AnalogClock = upper left edge of ConfigPanel"
	southEastVector := self extent // 2.
	northEastVector := Point x: (southEastVector x) y: (southEastVector y negated).
	analogClockCorners := Array with: (self center + southEastVector) with: (self center - southEastVector) with: (self center + northEastVector) with: (self center - northEastVector).
	"check for conflict with display area"
	candidatePositions := analogClockCorners select: [:each | Display boundingBox containsRect: (aRectangleMorph boundsInWorld translateBy: each)].
	candidatePositions 
		ifEmpty: [^ self center]
		ifNotEmpty: [^ candidatePositions first].
! !

!AnalogClock methodsFor: 'standalone-app' stamp: 'pv 7/7/2011 23:40'!
pvtSetAbilityToMove
	self class fullWorld ifFalse: [^ self].
	self setToAdhereToEdge: #adjustedCenter;
		beSticky! !

!AnalogClock methodsFor: 'geometry' stamp: 'pv 9/19/2011 22:13'!
radius
	^ self extent x / 2! !

!AnalogClock methodsFor: 'accessing' stamp: 'pv 8/20/2011 10:38'!
running: aBoolean
	viewController extension setProperty: #running toValue: aBoolean! !

!AnalogClock methodsFor: 'dropping/grabbing' stamp: 'pv 6/24/2011 13:08'!
wantsDroppedMorph: aMorph event: evt
	<lint: 'Sends "questionable" message' rationale: 'We need to know if the dropped morph responds to this message at least' author: 'Pascal Vollmer'>
	"Intentionally no call to super: the whole job is done here"
	"2011-05-24pv code unchanged although SwaLint-warning: isKindOf"
	^ (self dropEnabled & ((aMorph respondsTo: #text) | (aMorph respondsTo: #vertices)))
! !

Morph subclass: #AnalogClockViewController
	instanceVariableNames: 'dial watchhands dateText clockDistributor if configPanel myGeometry configuredWithOffset'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PVpub-AnalogClock'!

!AnalogClockViewController commentStamp: 'pv 3/16/2013 16:25' prior: 0!
An AnalogClockViewController is a submorph of clock which itself contains tickmarks and figures as submorphs.

Instance Variables
	clockDistributor 	<ClockDistributor>
	configPanel 		<ConfigPanel>
	configuredWithOffset <Boolean>
	dateText:			<DateText>
	if 					<AnalogClockInterface>
	myGeometry 		<Dictionary>
	watchhands 		<WatchHand>

clockDistributor 
	- control timing aspect
	
configPanel
	- let user configure
	
configuredWithOffset
	- recall whether user has asked for an time offset e.g. to test 
	the clock.
	
dateText
	- shows date, like 11 November
	
if
	- interface used by the configPanel
	
myGeometry
	- several dates related to my position
	
watchhands 
	- watchhands
	
Responsibilities <Collaborating classes>
	I embed all visible (moving and non-moving) parts of the clock as direct or indirect submorphs
	<WatchHand><DateMorph><Dial><Spiral>
		
!

!AnalogClockViewController class methodsFor: 'as yet unclassified' stamp: 'pv 6/19/2011 22:16'!
fractionOfTwoPiFrom: aNominator by: aDenominator
	^ Float pi * 2 asFloat * (Fraction numerator: aNominator denominator: aDenominator) asFloat! !

!AnalogClockViewController methodsFor: 'accessing' stamp: 'pv 8/20/2011 11:13'!
configPanel 
	^ configPanel ! !

!AnalogClockViewController methodsFor: 'accessing' stamp: 'pv 11/15/2011 23:33'!
configuredWithOffset
	"This array with 2 values is a time serie. Index 1 corresponds to
	the most recent value."
	configuredWithOffset at: 2 put: (configuredWithOffset first).
	configuredWithOffset at: 1 put: (self extension valueOfProperty: #dateAndTimeSource)
			= #osClockWithOffset.
	^ (configuredWithOffset first)! !

!AnalogClockViewController methodsFor: 'drawing' stamp: 'pv 11/1/2011 01:10'!
drawOn: aCanvas 
	| |
	self owner ifNil: [^ self]. 
	self syncMidnightShapeWith: self owner.
	super drawOn: aCanvas  ! !

!AnalogClockViewController methodsFor: 'accessing' stamp: 'pv 11/10/2011 21:33'!
getProperty: aPropertySymbol
	^ self extension valueOfProperty: aPropertySymbol ! !

!AnalogClockViewController methodsFor: 'drawing' stamp: 'pv 11/13/2011 13:15'!
hideDate
	dateText hide.
	dial showFigure3! !

!AnalogClockViewController methodsFor: 'drawing' stamp: 'pv 8/20/2011 12:53'!
hideTickmarks
	dial hideTickmarks! !

!AnalogClockViewController methodsFor: 'accessing' stamp: 'pv 9/18/2011 22:17'!
if
	"Abbreviation for AnalogClockInterface"
	^ if! !

!AnalogClockViewController methodsFor: 'initialization' stamp: 'pv 11/15/2011 23:31'!
initialize
	"self flag: #RQF005.	"
	"is categorized to parts bin since parent class uses same
	category and SmallLint does not want inconsistent method
	classification."
	super initialize.
	"we do fix the display area of the clock. but we do not show
	anything. Our submorphs only show themselves."
	self color: Color transparent.
	"remember current and last value of a user action."
	configuredWithOffset := Array with: false with: false.
	self pvtInitializeClockDistributor.
	self pvtInitializeGeometryAndSubmorphs.
	self pvtInitializeConfiguration; pvtInitializeInterface! !

!AnalogClockViewController methodsFor: 'initialization' stamp: 'pv 11/6/2011 01:45'!
initializeConfigPanel
	configPanel
		ifNil: [configPanel := ConfigPanel new.
			configPanel connectTo: if].
	configPanel
		position: (self pvtPlaceConfigPanelWhichIs: configPanel);
		openInWorld.
	self world doOneCycleNow! !

!AnalogClockViewController methodsFor: 'accessing' stamp: 'pv 11/14/2011 22:59'!
newlyConfiguredWithOffset
	^ (configuredWithOffset first)
		& ((configuredWithOffset second)
				~= (configuredWithOffset first))! !

!AnalogClockViewController methodsFor: 'initialization' stamp: 'pv 8/15/2011 22:42'!
pvtInitializeClockDistributor
	clockDistributor := ClockDistributor new.
	clockDistributor
		color: (self color alpha: 0.0).
	self addMorph: clockDistributor! !

!AnalogClockViewController methodsFor: 'initialization' stamp: 'pv 8/20/2011 10:36'!
pvtInitializeConfiguration
	self assureExtension 
		setProperty: #configPanelOn toValue: false;
		 setProperty: #spiraClockOn toValue: false;
		 setProperty: #tickmarksOn toValue: true;
		 setProperty: #dateOn toValue: true;
		 setProperty: #dateAndTimeSource toValue: #osClock;
		 setProperty: #running toValue: true! !

!AnalogClockViewController methodsFor: 'initialization' stamp: 'pv 11/15/2011 23:56'!
pvtInitializeDate
	DateText initialize.
	"initialize class variables"
	dateText := DateText new.
	dateText contents: 'mm/dd/yyyy';
		 backgroundColor: Color white;
		 color: Color black;
		 borderWidth: 1;
		 borderColor: Color blue;
		 borderRaised;
		 useRoundedCorners;
		
		font: (myGeometry at: #font);
		
		setTextStyle: (myGeometry at: #textStyle);
		
		setTimeMessage: #asDate
		divisor: 1
		andSize: 0.3.
	"setFont: font andTextStyle: textStyle;"
	self addMorph: dateText! !

!AnalogClockViewController methodsFor: 'initialization' stamp: 'pv 11/6/2011 01:31'!
pvtInitializeDial
	dial := Dial new.
	dial initializeSubmorphs.
	self addMorph: dial.
	dial syncMidnightShapeWith: self! !

!AnalogClockViewController methodsFor: 'initialization' stamp: 'pv 3/16/2013 19:38'!
pvtInitializeGeometryAndSubmorphs 
	myGeometry := Dictionary new.
	myGeometry at: #defaultMidnightShape put: true;
		 at: #ownersCenter put: 0;
		 at: #ownersRadius put: 0;
		"shall be common to all submorphs which contain text"
		at: #textStyle put: TextStyle default copy.
	(myGeometry at: #textStyle) alignment: 2. "text centered"
	myGeometry at: #font put: (StrikeFont familyName: 'BitstreamVeraSans' size: 16).		
	"(z-order) Spiral is under Dial, Dial is under WatchHands."
	watchhands := Dictionary new.	
	self pvtInitializeDial.
	self pvtInitializeSpiral.
	self pvtInitializeDate.
	self pvtInitializeWatchHands.
! !

!AnalogClockViewController methodsFor: 'initialization' stamp: 'pv 11/13/2011 12:57'!
pvtInitializeInterface
	"let AnalogClock get steered by AnalogClockInterface"
	if := AnalogClockInterface new.
	if watchhands: watchhands.
	if clockDistributor: clockDistributor.
	if viewController: self! !

!AnalogClockViewController methodsFor: 'initialization' stamp: 'pv 11/15/2011 22:28'!
pvtInitializeSpiral
	Spiral initialize. "initialize class variables"
	watchhands at: #Spiral put: Spiral new.
	(watchhands at: #Spiral)
		setTimeMessage: #minutes divisor: 60	andSize: 1.0;
		syncMidnightShapeWith: self.
	self addMorph: (watchhands at: #Spiral).	
	(watchhands at: #Spiral) hide.! !

!AnalogClockViewController methodsFor: 'initialization' stamp: 'pv 11/15/2011 23:27'!
pvtInitializeWatchHands
	watchhands at: #Hour put: WatchHand new.
	(watchhands at: #Hour)
		setTimeMessage: #hours
		divisor: 12
		andSize: 0.65;
		 color: Color blue;
		 borderColor: Color blue;
		 syncMidnightShapeWith: self.
	watchhands at: #Min put: WatchHand new.
	(watchhands at: #Min)
		setTimeMessage: #minutes
		divisor: 60
		andSize: 0.85;
		 color: Color blue;
		 borderColor: Color blue;
		 syncMidnightShapeWith: self.
	watchhands at: #Sec put: WatchHand new.
	(watchhands at: #Sec)
		setTimeMessage: #seconds
		divisor: 60
		andSize: 0.95;
		 color: Color blue;
		 borderColor: Color blue;
		 syncMidnightShapeWith: self.
	self
		addMorph: (watchhands at: #Hour);
		addMorph: (watchhands at: #Min);
		addMorph: (watchhands at: #Sec)! !

!AnalogClockViewController methodsFor: 'converting' stamp: 'pv 7/8/2011 21:41'!
pvtOffsetVectorFromRadius: aFloat withRelativeLength: relLength forIndex:  index ofTotal: total
	"080413pv without rounded this would yield float-bounds which in turn will lead to pixel errors."
	| f x y |
	f :=	self class fractionOfTwoPiFrom: index by: total.   
	x := aFloat * relLength * f sin.
	y := 0 - (aFloat * relLength * f cos).
	^ x rounded @ y rounded! !

!AnalogClockViewController methodsFor: 'geometry' stamp: 'pv 8/20/2011 11:11'!
pvtPlaceConfigPanelWhichIs: aRectangleMorph
	| southEastVector northEastVector analogClockCorners candidatePositions |
	"default pos: lower right edge of AnalogClock = upper left edge of ConfigPanel"
	southEastVector := self extent // 2.
	northEastVector := Point x: (southEastVector x) y: (southEastVector y negated).
	analogClockCorners := Array with: (self center + southEastVector) with: (self center - southEastVector) with: (self center + northEastVector) with: (self center - northEastVector).
	"check for conflict with display area"
	candidatePositions := analogClockCorners select: [:each | Display boundingBox containsRect: (aRectangleMorph boundsInWorld translateBy: each)].
	candidatePositions 
		ifEmpty: [^ self center]
		ifNotEmpty: [^ candidatePositions first].
! !

!AnalogClockViewController methodsFor: 'geometry' stamp: 'pv 8/18/2011 11:40'!
radius
	^ self extent x / 2! !

!AnalogClockViewController methodsFor: 'initialization' stamp: 'pv 8/20/2011 12:17'!
removeConfigPanel
	configPanel delete.
	configPanel := nil.
! !

!AnalogClockViewController methodsFor: 'drawing' stamp: 'pv 8/20/2011 15:48'!
rotateLazyAt: dateAndTime
	"my submorphs are concerned only. I do not have anything to show."
	self submorphsDo: 
		[ :each | each rotateLazyAt: dateAndTime ].  ! !

!AnalogClockViewController methodsFor: 'accessing' stamp: 'pv 8/20/2011 10:41'!
running
	^ self extension valueOfProperty: #running! !

!AnalogClockViewController methodsFor: 'accessing' stamp: 'pv 8/20/2011 10:41'!
running: aBoolean
	self setProperty: #running toValue: aBoolean! !

!AnalogClockViewController methodsFor: 'geometry' stamp: 'pv 9/19/2011 22:03'!
sameR: r andSameC:  c 
	^ ((myGeometry at: #ownersRadius) = r) & ((myGeometry at: #ownersCenter) = c)! !

!AnalogClockViewController methodsFor: 'drawing' stamp: 'pv 11/13/2011 13:15'!
showDate
	dateText show.
	dial hideFigure3! !

!AnalogClockViewController methodsFor: 'drawing' stamp: 'pv 8/20/2011 12:53'!
showTickmarks
	dial showTickmarks! !

!AnalogClockViewController methodsFor: 'accessing' stamp: 'pv 11/13/2011 15:08'!
spiraClockOn
	^ (extension valueOfProperty: #spiraClockOn)! !

!AnalogClockViewController methodsFor: 'geometry' stamp: 'pv 11/13/2011 13:15'!
syncMidnightShapeWith: aMorph 
	| r c |
	r := aMorph radius rounded.
	c := aMorph center.
	(self sameR: r andSameC: c)
		ifTrue: [^ self].
	self
		bounds: (c - (r @ r) corner: c + (r @ r)).
	myGeometry at: #ownersCenter put: c;
		 at: #ownersRadius put: r.
	"dateText takes the responsibility to hide itself.
	However it needs our trigger to show itself again
	when the bounds of AnalogClock get larger again."
	dateText syncMidnightShapeWith: self! !

!AnalogClockViewController methodsFor: 'accessing' stamp: 'pv 8/20/2011 11:53'!
toggleConfigPanel
	(self extension valueOfProperty: #configPanelOn)
		ifTrue:	[self removeConfigPanel]
		ifFalse: 	[self initializeConfigPanel].
	self extension setProperty: #configPanelOn toValue: (self extension valueOfProperty: #configPanelOn) not! !

!AnalogClockViewController methodsFor: 'accessing' stamp: 'pv 10/31/2011 23:43'!
toggleDate
	"071112pv this method is triggered when someone is using
	the Halo-menu as specified in addCustomMenuItems:hand:"
	(extension valueOfProperty: #dateOn)
		ifTrue: [self hideDate]
		ifFalse: [self showDate].
	extension setProperty: #dateOn toValue: (extension valueOfProperty: #dateOn) not! !

!AnalogClockViewController methodsFor: 'converting' stamp: 'pv 9/18/2011 22:11'!
toggleSpiraClock
	"071112pv this method is triggered when someone is using the Halo-menu as specified in addCustomMenuItems:hand:"
	| spiral |
	
	spiral := watchhands at: #Spiral.
	(extension valueOfProperty: #spiraClockOn) 
		ifTrue: 
			[spiral hide]
		ifFalse:	
			[spiral syncMidnightShapeWith: self.
			spiral show].
	extension setProperty: #spiraClockOn toValue: (extension valueOfProperty: #spiraClockOn) not! !

!AnalogClockViewController methodsFor: 'accessing' stamp: 'pv 9/18/2011 22:13'!
toggleTickmarks
	"071112pv this method is triggered when someone is using the Halo-menu as specified in addCustomMenuItems:hand:"
	(extension valueOfProperty: #tickmarksOn) 
		ifTrue: [dial hideTickmarks]
		ifFalse: [dial showTickmarks].
	extension setProperty: #tickmarksOn toValue: (extension valueOfProperty: #tickmarksOn) not! !

!AnalogClockViewController methodsFor: 'halos and balloon help' stamp: 'pv 8/31/2010 18:14'!
wantsHaloFromClick
	^false! !

!AnalogClockViewController methodsFor: 'accessing' stamp: 'pv 8/20/2011 07:06'!
watchhandSec
	^ watchhands at: #Sec! !

!AnalogClockViewController methodsFor: 'accessing' stamp: 'pv 11/13/2011 12:58'!
watchhands 
	^ watchhands! !

Object subclass: #AnalogClockDocumentation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PVpub-AnalogClock'!

!AnalogClockDocumentation commentStamp: 'pv 3/17/2013 20:53' prior: 0!
<?xml version="1.0" encoding="UTF-8"?>
<article version="5.0"
	 xmlns:xlink="http//www.w3.org/1999/xlink"
	 xmlns="http://docbook.org/ns/docbook">
  <title>Project Manual for AnalogClock</title>
  <para>Why something different from MorphicExtras-Demo/WatchMorph ?</para>
  <itemizedlist spacing="compact">
    <listitem>
      <para>Because I needed a general purpose tutorial for my course (children from 12 to 16).</para>
    </listitem>
    <listitem>
      <para>Because I wanted to check non-standard behavior with Morphic like non-rectangular windows and animated submorphs.</para>
    </listitem>
    <listitem>
      <para>Because I wanted to have the watchhands as PolygonMorphs so that they could be designed by the user in an individual manner.</para>
    </listitem>
    <listitem>
      <para>Because I was fond of the SpiraClock idea (http://www.emn.fr/x-info/spiraclock/) and I wanted to migrate it to Squeak.</para>
    </listitem>
  </itemizedlist>
  <para>Reviews are welcome!! I am a Squeak newbie, I am sure I still make a lot of mistakes and I would like to learn more. If you find a bug in design or implementation, please email me at pascal.vollmer@ieee.org. I will try to correct, I will send you 1 Dollar or 1 EUR depending on where you live and I will acknowledge your contribution to this tutorial.</para>

  <section>
    <title>How to download</title>
    <para>From SqueakSource please download</para>
    <orderedlist>
      <listitem><para>ical</para></listitem>
      <listitem><para>AnalogClock.</para></listitem>
    </orderedlist>
    <para>
Then apply a patch to ical. Modify the class method of ICCalendarVersion named value.
<programlisting>
value
	"090101pv squeak does not know raiseSignal"
	"^ self raiseSignal"
	^ self signal</programlisting>
    </para>
  </section>

  <section>
    <title>How to invoke the application</title>
    <para>
AnalogClock new openInWorld - shows an analog clock, much like MorphicExtras-Demo/WatchMorph.

The clock is composed of several submorphs. It is movable and resizable.

It is possible to map calendar events to the dial area. We use here the idea of Spiraclock (see comment in Spiral).

In order to grasp the function of a SpiralCalendar: 
1) run the tests (TestRunner open and select AnalogClockACceptanceTest and AnalogClockUnitTest - exclude AnalogClockTest) or
2) take an event from your calendar (I test with Linux/KDE/Kalendar and with Windows/Outlook), copy event data in ical-format. Then paste them into a TextMorph (from Supplies flap or from World/objects). Drag the TextMorph with the "Pick up"-knob and drop it onto the AnalogClock. Event needs to be in the near future, let's say 1 hour ahead. The event is then shown in the spiral. It's shape is updated each minute. As soon as the minute watchhand reaches the beginning of the event, the representation will show the remaining time.
    </para>
  </section>

  <section>
    <title>Requirements</title>
    <para>Each requirement has a unique identifier like 'RQF001'. The abbreviation RQF is for functional requirements. RQN is for non-functional requirements. 
We try to flag methods with the requirements they implement. This is done following the proposal from http://wiki.squeak.org/squeak/381. In order to find the (most important) methods which implement a requirement select the requirement's identifier, type cmd-n and browse the list.
Find in parenthesis after the RQxyyy the version number where the requirement is planned to be implemented. If there is more than one version number, replanning has taken place. Find in parenthesis at the end the version number where the requirement has been implemented. "R2" for instance is for the monticello package with the suffix "*.2". For the version-plan see 3.</para>
<section>
    <title>Functional Requirements</title>
    <itemizedlist spacing="compact">
      <listitem>
	<para><indexterm type="RQF"><primary>RQF001</primary></indexterm>RQF001 <xref linkend="VER001"/>Show an analogue clock with watchhands for hours, minutes and seconds.<xref linkend="VER001"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF002</primary></indexterm>RQF002 <xref linkend="VER001"/>Let the watchhands be polygons.<xref linkend="VER001"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF003</primary></indexterm>RQF003 <xref linkend="VER001"/>Let the clock be movable and resizable.<xref linkend="VER001"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF004</primary></indexterm>RQF004 <xref linkend="VER001"/>Let it show the current time.<xref linkend="VER001"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF005</primary></indexterm>RQF005 <xref linkend="VER002"/>Show a dial with figures<xref linkend="VER002"/></para>
      </listitem>
      <listitem>
	<para>RQF006 --empty--</para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF007</primary></indexterm>RQF007 <xref linkend="VER004"/>Default shape of the watchhands is a lozenge.<xref linkend="VER004"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF008</primary></indexterm>RQF008 <xref linkend="VER005"/>Put a spiral on the dial for Spiraclock functionality.<xref linkend="VER005"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF009</primary></indexterm>RQF009 <xref linkend="VER006"/>Place a SpiralEvent in the spiral.<xref linkend="VER006"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF010</primary></indexterm>RQF010 <xref linkend="VER007"/>Display the past part of a SpiralEvent in a different way (e.g. display nothing for the past part) <xref linkend="VER007"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF011</primary></indexterm>RQF011 <xref linkend="VER012"/>Test for dial figures without pixel errors - see <xref linkend="BUG006"/> <xref linkend="VER012"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF012</primary></indexterm>RQF012 <xref linkend="VER007"/>Import an iCalendar event from Linux/KDE/KOrganizer or newer: Linux/KDE/Kontact to a TextMorph and drop this TextMorph on us.<xref linkend="VER007"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF013</primary></indexterm>RQF013 <xref linkend="VER014"/>Change the alpha-value as an event moves from future to past. >>>>>20111101 meaning not clear. <xref linkend="VER014"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF014</primary></indexterm>RQF014 <xref linkend="VER014"/>Jitter of the shown duration of a second on a full screen clock should not be more than +/-100% i.e. duration should be between 0ms and 2000ms. <xref linkend="VER014"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF015</primary></indexterm>RQF015 <xref linkend="VER008"/>SpiraClock-feature may be switched on or off via Halo menu <xref linkend="VER008"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF016</primary></indexterm>RQF016 <xref linkend="VER008"/>Import multiple events like in RQF012.<xref linkend="VER008"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF017</primary></indexterm>RQF017 <xref linkend="VER007"/>While AnalogClock is moved or resized all submorphs are hidden.<xref linkend="VER007"/>Now canceled: see RQF024</para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF018</primary></indexterm>RQF018 <xref linkend="VER009"/>z-Sequence of visual components is (from bottom to top): background - figures - tickmarks - spiral - spiral-event - date - hour-watchhand - minute-watchhand - second-watchhand.<xref linkend="VER009"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF019</primary></indexterm>RQF019 <xref linkend="VER010"/>AnalogClock may be duplicated several times and all clocks will work correctly.<xref linkend="VER010"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF020</primary></indexterm>RQF020 <xref linkend="VER014"/>If new event overlaps with already accepted one: let user decide who is the winner. Now canceled: see RQF063<xref linkend="VERxxx"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF021</primary></indexterm>RQF021 <xref linkend="VER012"/>Let the clock show: (current time + an offset defined by the user). Give the user a button to increment/decrement/reset this offset. Show the effect immediately. If an increment/decrement operation for a time period is active freeze the watchhands of all smaller time periods. Example: if the user wants to decrement by minutes the watchhand for seconds get frozen.<xref linkend="VER012"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF022</primary></indexterm>RQF022 <xref linkend="VER014"/>SpiraClock on: for each event show its caption. Alternative: for each event and with mouse hovering on it show its caption.<xref linkend="VER014"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF023</primary></indexterm>RQF023 <xref linkend="VER014"/>Let the watchhands be Morphs (submorphs to the clock) and prepare to get them dragged and dropped on the clock.<xref linkend="VER014"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF024</primary></indexterm>RQF024 <xref linkend="VER012"/>While AnalogClock is moved or resized all submorphs are moved or resized. (was RQF017)<xref linkend="VER012"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF025</primary></indexterm>RQF025 <xref linkend="VER013"/>While resizing AnalogClock to smaller size the spiral segment showing an event may be dropped. Make sure it is shown again when resizing back to larger.<xref linkend="VER013"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF026</primary></indexterm>RQF026 <xref linkend="VER017"/>Create several AnalogClock instances with the duplicate button in the halo. Drop an event on one of them. The other instances shall remain unchanged.<xref linkend="VER032"/></para>
      </listitem>
    </itemizedlist>
<para>---everything resolved above this line---</para>
  <itemizedlist spacing="compact">
      <listitem>
	<para><indexterm type="RQF"><primary>RQF027</primary></indexterm>RQF027 <xref linkend="VER017"/>Show the current time as a string. Let this be configurable.<xref linkend="VERxxx"/></para>
      </listitem>
      <listitem>
	<para>RQF028 (VER017) Show the current date as a string like mm/dd/yyyy on the dial. Let this be configurable to show or hide.</para>
      </listitem>
      <listitem>
	<para>RQF029 (VER014) Show tickmarks. Let this be configurable to show or hide. (VER014)</para>
      </listitem>
      <listitem>
	<para>RQF030 (VER014) When a configuration panel has been created and the clock is deleted the configuration panel should get deleted with it. (VER014)</para>
      </listitem>
      <listitem>
	<para>RQF031 (VER021) As the figures from 1 to 12 are displayed, use their double meaning as hours and months. Show the current date and display a day field next to the figure which represents the current month. Let this be configurable to show or hide.</para>
      </listitem>
      <listitem>
	<para>RQF032 (VER017) For each watchhand let it be configurable whether it moves continuously or in discrete steps. To give an example, the clocks of "Deutsche Bahn" in Germany move like this: second-continuously, minutes-discrete, hours-continuously.</para>
      </listitem>
      <listitem>
	<para>RQF033 (VER020) Localize DateMorph.</para>
      </listitem>
      <listitem>
	<para>RQF034 (VER016) Add little tickmarks for seconds. (VER016) </para>
      </listitem>
      <listitem>
	<para>RQF035 (VER016) Add a start/stop button in the ConfigPanel. Stop stops every movement of watchhands. (VER016)</para>
      </listitem>
      <listitem>
	<para>RQF036 (VER017) Allow custom made PolygonMorphs to be dropped to replace the default watchhands. (Stop the clock, make sure the rotation center of the new PolygonMorph is well placed, drag the new watchhand over the replaced watchhand and drop) (VER017)</para>
      </listitem>
      <listitem>
	<para>RQF037 (VER019) Create a Windows executable</para>
      </listitem>
      <listitem>
	<para>RQF038 (VER019) Prevent user from rotating the clock </para>
      </listitem>
      <listitem>
	<para>RQF039 (VER016) Show the current date as dd Month on the dial. Let this be configurable to show or hide. (VER016)</para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF040</primary></indexterm>RQF040 <xref linkend="VER017"/> Prepare to be a standalone application (following Steve Wessels Laser game tutorial). <xref linkend="VER034"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF041</primary></indexterm>RQF041 <xref linkend="VER022"/> Add the OS-specific things for a standalone application and test on a Linux machine. <xref linkend="VER034"/></para>
      </listitem>
      <listitem>
	<para>RQF042 (VER024) ConfigPanel could be a circle and the Buttons could be segments of the circle (like in cpntools)</para>
      </listitem>
      <listitem>
	<para>RQF043 (VER024) AnalogClock could be used to visualize multidimensional time series like in a three-dimensional star diagram , the z-axis being the time-axis. If values are at least interval scale and if normal distribution can be seen as an appropriate mean to score the values this would mean that r/2 values are optimal ("green"). Values at 0 or r would be non-optimal ("red").</para>
      </listitem>
      <listitem>
	<para>RQF044 (VER024) iCal-data can be dropped on AnalogClock. Provide the reverse operation: drag SpiralEvent from AnalogClock and transform it to a TextMorph containing the iCal-data.</para>
      </listitem>
      <listitem>
	<para>RQF045 (VER024) add Buttons for the size of the time increment to ConfigPanel (VER024)</para>
      </listitem>
      <listitem>
	<para>RQF046 (VER024) draw border of DateMorph with same distance from bounding box of the text horizontally as vertically. (VER025)</para>
      </listitem>
      <listitem>
	<para>RQF047 (VER028) (just for fun) let the clock move horizontally according to the current center of mass of the three watchhands</para>
      </listitem>
      <listitem>
	<para>RQF048 (VER028) (just for fun) let one watchhand stick to a fixed position (instead of the dial), for instance to south. Make this watchhand longer as the dial radius. Let the clock rotate around this fixed watchhand.</para>
      </listitem>
      <listitem>
	<para>RQF049 (VER028) let some pictures appear as dial background as the time passes. Examples: Photographs, documents to be used for a meeting.</para>
      </listitem>
      <listitem>
	<para>RQF050 (VER028) import from Microsoft Outlook ("export as iCalendar") (VER028)</para>
      </listitem>
      <listitem>
	<para>RQF051 (VER034) treat timezone and write a corresponding identifier (e.g. "London") above the clock's center</para>
      </listitem>
      <listitem>
	<para>RQF052 (VER045) let the seconds whatchhand move continuously (configurable)</para>
      </listitem>
      <listitem>
	<para>RQF053 (VER054) allow an AnalogClock to get embedded in an AnalogClock (as it is done in chronographs for example, refer to RQF026)</para>
      </listitem>
      <listitem>
	<para>RQF054 (VER057) add three buttons to the configuration panel in order to show/hide the second, minute, hour watchhand. (VER058)</para>
      </listitem>
      <listitem>
	<para>RQF055 (VER062) add an internal counter to sum up the test and usage time of a version of the AnalogClock. The counter is reset to zero when a new version is deployed. This is the base for MTBF calculation (MTBF=mean time between failure). (VERxxx)</para>
      </listitem>
      <listitem>
	<para>RQF056 (VER062) add assertions. When an assertion is hit record the duration since the last assertion violation. This will help to build MTBF statistics. (VERxxx)</para>
      </listitem>
      <listitem>
	<para>RQF057 (VER062) add a halo menu entry for the user to record a problem. Imagine the user encounters a problem and the ui is still alive. Encourage the user to go to the halo menu and to enter "hey, I encountered a problem". This would be associated with a time stamp. We would then know how much time passed since the last problem. And this would help to build MTBF statistics. (VERxxx)</para>
      </listitem>
      <listitem>
	<para>RQF058 (VER062) when you set seconds, minutes or hours via the ConfigPanel make sure that watchhands with higher resolution get freezed. (VERxxx)</para>
      </listitem>
      <listitem>
	<para>RQF059 (VER063) When being a.m. ("ante meridiam") show figures from 1 to 12, when being p.m. ("post meridiam") show figures from 13 to 24(VERxxx)</para>
      </listitem>
      <listitem>
	<para>RQF060 (VER063) When you shrink the size of the analog clock there is one moment where a date is not shown any more. Increase the size again: the date should be shown again. (VERxxx)</para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF061</primary></indexterm>RQF061<xref linkend="VERxxx"/>Express memory and computing time as a function of the number of imported events like in RQF016.<xref linkend="VERxxx"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF062</primary></indexterm>RQF062 <xref linkend="VERxxx"/>Express memory and computing time as a function of the number of duplicated clocks.<xref linkend="VERxxx"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF063</primary></indexterm>RQF063 <xref linkend="VER064"/>If new event overlaps with already accepted one: event which comes later is shown on top of event which occurs earlier<xref linkend="VERxxx"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF064</primary></indexterm>RQF064<xref linkend="VER064"/>Let user decide whether labels for events can be edited at runtime or not.<xref linkend="VERxxx"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF065</primary></indexterm>RQF065<xref linkend="VER064"/>Synchronize with Linux Calendar application (e.g. Kontact).<xref linkend="VERxxx"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF067</primary></indexterm>RQF067<xref linkend="VERxxx"/>Port to Cuis (and check anti-aliasing).<xref linkend="VERxxx"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF068</primary></indexterm>RQF068<xref linkend="VERxxx"/>Add user information to the AnalogClockAcceptanceTest. For instance: "We now calculate jitter, please wait 100seconds. We now present the jitter value: xxx." or "We show an event in the spiral, are you satisfied with the layout? yes/no."<xref linkend="VERxxx"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQF"><primary>RQF066</primary></indexterm>RQF066<xref linkend="VERxxx"/>new requirement.<xref linkend="VERxxx"/></para>
      </listitem>
    </itemizedlist>
  </section>
  <section>
    <title>Non-Functional Requirements</title>
    <itemizedlist spacing="compact">
      <listitem>
	<para><indexterm type="RQN"><primary>RQN001</primary></indexterm>RQN001 remove Transcript show: -messages<literal role="REL">(VER003)</literal></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQN"><primary>RQN002</primary></indexterm>RQN002 (design to test) create fast motion of watchhands to test progressing/degressing time<literal role="REL">(VER012)</literal></para>
      </listitem>
      <listitem>
	<para>RQN003 (documentation) make this project manual visible maybe by using LatexMorph, maybe embedded into what Squeak calls a "project", maybe in a comment.(VER014,VER017,VER036)</para>
      </listitem>
    </itemizedlist>
<para>---everything resolved above this line---</para>
  <itemizedlist spacing="compact">
      <listitem>
	<para>RQN004 (design) create an abstract "moveable and resizable animated shape" and let all the watchhands, dials, spirals etc. inherit from it.</para>
      </listitem>
      <listitem>
	<para>RQN005 (performance) Preferences/debugShowDamage shows 3 damaged regions per execution of step method, one per watchhand. But there should be damaged regions only for watchhands which move.(VER012)</para>
      </listitem>
      <listitem>
	<para>RQN006 (design) (VER034) let "configuration" be a class of its own.(VER055)</para>
      </listitem>
      <listitem>
	<para><indexterm type="RQN"><primary>RQN007</primary></indexterm>RQN007 <xref linkend="VER040"/> Link DocBook documentation to the image. For instance let a requirement point to the method(s) which implement it.</para>
      </listitem>
      <listitem>
	<para><indexterm type="RQN"><primary>RQN008</primary></indexterm>RQN008 <xref linkend="VER040"/> Use AsciiDoc instead of DocBook to keep the project manual readable.</para>
      </listitem>
      <listitem>
	<para><indexterm type="RQN"><primary>RQN009</primary></indexterm>RQN009 <xref linkend="VER053"/> Provide means for jitter measurement on seconds watchhand. This is to have an objective figure for the sensation of "stumbling" of the second watchhand.<xref linkend="VER053"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQN"><primary>RQN010</primary></indexterm>RQN010 <xref linkend="VER064"/>Write down a testplan which checks orthogonality of configurations an operations. Configurations: with/without spiral, with/without date, etc. Operations: add spiraclock event, add watchhand, etc.<xref linkend="VERxxx"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQN"><primary>RQN011</primary></indexterm>RQN011 <xref linkend="VERxxx"/>new non-functional requirement.<xref linkend="VERxxx"/></para>
      </listitem>
  </itemizedlist>
    </section>
  <section>
    <title>Process and Tools Requirements</title>
    <itemizedlist spacing="compact">
      <listitem><para></para></listitem>
    </itemizedlist>
<para>---everything resolved above this line---</para>
  <itemizedlist spacing="compact">
      <listitem>
	<para><indexterm type="RQP"><primary>RQP001</primary></indexterm>RQP001 Document requirements, versions and bugs. Document the links between them.</para>
      </listitem>
      <listitem>
	<para><indexterm type="RQP"><primary>RQP002</primary></indexterm>RQP002 Include quality information in version information.<xref linkend="VER059"/></para>
      </listitem>
      <listitem>
	<para><indexterm type="RQP"><primary>RQP003</primary></indexterm><xref linkend="VER063"/>RQP003 Check whether pragmas can be used to express link between method and requirement.</para>
      </listitem>
      <listitem>
	<para><indexterm type="RQP"><primary>RQP004</primary></indexterm><xref linkend="VERxxx"/>Create a glossary fo this Project Manual. Create means to insert glossary terms into the text in a way that they get linked to the explanation in the glossary.</para>
      </listitem>
      <listitem>
	<para><indexterm type="RQP"><primary>RQP005</primary></indexterm><xref linkend="VERxxx"/>new process requirement.</para>
      </listitem>
  </itemizedlist>
    </section>
  </section>

  <section>
    <title>Design</title>
    <para>Design issues are collected here.</para>
    <itemizedlist spacing="compact">
      <listitem xml:id="ResizeableDateField" xreflabel="(Resizeable date field)">
	<para>When the clock is resized the date field shall be resized as well. Two situations here: 1. not enough space to display the date text -> hide, 2. enough space to display -> show.
	If we are in the second state we shall determine a font size that leaves the date text field smaller than the radius of the clock.
	</para>
      </listitem>
    </itemizedlist>
  </section>

  <section>
    <title>Implementation</title>
    <para>This is just a placeholder. Later on we want to generate this section from Squeak automatically.</para>
    <itemizedlist spacing="compact">
      <listitem xml:id="AnalogClockViewController" xreflabel="(AnalogClockViewController)">
	<para>AnalogClockViewController</para>
	<itemizedlist spacing="compact">
	  <listitem>
	    <para>initialize</para>
	  </listitem>
	</itemizedlist>
      </listitem>
      <listitem xml:id="AnalogClockTest" xreflabel="(AnalogClockTest)">
	<para>AnalogClockTest</para>
	<itemizedlist spacing="compact">
	  <listitem>
	    <para>setup</para>
	  </listitem>
	  <listitem>
	    <para>tearDown</para>
	  </listitem>
	</itemizedlist>
      </listitem>
  </itemizedlist>
  </section>

  <section>
    <title>Versions and Quality</title>
    <itemizedlist spacing="compact">
      <listitem xml:id="VER001" xreflabel="(VER001)">
	<para><indexterm type="VER"><primary>VER001</primary></indexterm><literal>VER001</literal> (PVpub-pv.1.mcz, 24 March 2007 4:23:18 pm) 
SmallLint: bugs=0
tests passed: 1/1</para>
      </listitem>
      <listitem xml:id="VER002" xreflabel="(VER002)">
	<para><indexterm type="VER"><primary>VER002</primary></indexterm><literal>VER002</literal> (PVpub-AnalogClock.2.mcz, 4 April 2007 11:24:54 pm)
SmallLint: bugs=0, possible bugs=4
tests passed: 1/1</para>
      </listitem>
      <listitem xml:id="VER003" xreflabel="(VER003)">
	<para>VER003 (PVpub-AnalogClock.3.mcz, 6 April 2007 12:40:18 am)
SmallLint: bugs=0, possible bugs=4
tests passed: 1/1
</para>
      </listitem>
      <listitem xml:id="VER004" xreflabel="(VER004)">
	<para>VER004 (PVpub-AnalogClock.4.mcz, 9 April 2007 7:12:10 pm)
SmallLint: bugs=0, possible bugs=4
tests passed: 1/1
</para>
      </listitem>
      <listitem xml:id="VER005" xreflabel="(VER005)">
	<para>VER005 (PVpub-AnalogClock.5.mcz, 5 May 2007 11:53:08 pm)
SmallLint: bugs=0, possible bugs=6
tests passed: 1/1
</para>
      </listitem>
      <listitem xml:id="VER006" xreflabel="(VER006)">
	<para>VER006 (PVpub-AnalogClock.6.mcz, 2 June 2007 7:37:07 pm)
SmallLint: bugs=0, possible bugs=6
tests passed: 1/1</para>
      </listitem>
      <listitem xml:id="VER007" xreflabel="(VER007)">
	<para>VER007  (PVpub-AnalogClock.7.mcz, 23 October 2007 5:07:50 pm +
	 PVpubTests-AnalogClock.7.mcz, 23 October 2007 5:11:15 pm)
SmallLint: bugs=0, possible bugs=6
tests passed: 1/1
</para>
      </listitem>
      <listitem xml:id="VER008" xreflabel="(VER008)">
	<para>VER008  (PVpub-AnalogClock.8.mcz, 14 November 2007 2:34:04 +
	 PVpubTests-AnalogClock.8.mcz, 14 November 2007 2:38:06)
SmallLint: bugs=0, possible bugs=6
tests passed: 1/1</para>
      </listitem>
      <listitem xml:id="VER009" xreflabel="(VER009)">
	<para>VER009  (PVpub-AnalogClock.9.mcz) 16 November 2007 16:04:42
SmallLint: bugs=0, possible bugs=2
tests passed: 1/1
</para>
      </listitem>
      <listitem xml:id="VER010" xreflabel="(VER010)">
	<para>VER010  (PVpub-AnalogClock.10.mcz, 19 November 2007 17:45:59 +
	 PVpubTests-AnalogClock.10.mcz, 19 November 2007 17:48:07)
SmallLint: bugs=0, possible bugs=2
tests passed: 2/2
</para>
      </listitem>
      <listitem xml:id="VER011" xreflabel="(VER011)">
	<para>VER011  (PVpub-AnalogClock.11.mcz, 20 December 2007 00:48:36 +
	 PVpubTests-AnalogClock.11.mcz, 20 December 2007 00:51:34)
SmallLint: bugs=0, possible bugs=2
tests passed: 2/2
review against K. Beck, SBPP, Chapter 7: ok</para>
      </listitem>
      <listitem xml:id="VER012" xreflabel="(VER012)">
	<para>VER012  (PVpub-AnalogClock.12.mcz, 15 April 2008 22:30:08)
SmallLint: bugs=0, possible bugs=3
tests passed: 2/2
review against K. Beck, SBPP, Chapter 7: ok
Summary: refactoring of morph/submorph dependency with respect to geometry</para>
      </listitem>
      <listitem xml:id="VER013" xreflabel="(VER013)">
	<para>VER013  (PVpub-AnalogClock.13.mcz, 2 May 2008, 21:33:06)
SmallLint: bugs=0, possible bugs=4
tests passed: 2/2
review against K. Beck, SBPP, Chapter 7: ok
Summary: some bugfixes
</para>
      </listitem>
      <listitem xml:id="VER014" xreflabel="(VER014)">
	<para>VER014  (PVpub-AnalogClock.14.mcz, 3 May 2008, 17:32:18)
all methods classified
SmallLint: bugs=0, possible bugs=4
tests passed: 2/2
review against K. Beck, SBPP, Chapter 7: ok
Summary: RQF029,RQF030, small fixes</para>
      </listitem>
      <listitem xml:id="VER015" xreflabel="(VER015)">
	<para>VER015  (PVpub-AnalogClock.15.mcz, 6 May 2008, 12:37:53)
all methods classified
SmallLint: bugs=0, possible bugs=5
tests passed: 2/2
review against K. Beck, SBPP, Chapter 7: ok
Summary: RQF028</para>
      </listitem>
      <listitem xml:id="VER016" xreflabel="(VER016)">
	<para>VER016  (PVpub-AnalogClock.16.mcz, 26 May 2008, 22:1x:xx)
all methods classified
SmallLint: bugs=0, possible bugs=6, squeak bugs=11, intention revealing=0, unnecessary code=1, miscellaneous=19
SwaLint: LongMethods=29, Bad message=2, Debugging code left in methods=11, Uses (to:)do: instead of to:do:=3
LOC-instance=771, LOC-class=3
tests passed: 2/2
review against K. Beck, SBPP, Chapter 7: ok
Summary: RQF034,RQF035,RQF039</para>
      </listitem>
      <listitem xml:id="VER017" xreflabel="(VER017)">
	<para>VER017  (PVpub-AnalogClock.17.mcz, 11 June 2008, 10:37:10 pm)
intermediate version, no Quality Assurance.
Summary: RQF040</para>
      </listitem>
      <listitem xml:id="VER018" xreflabel="(VER018)">
	<para>VER018  (PVpub-AnalogClock.18.mcz, 11 June 2008, 10:58:20 pm)
intermediate version, no Q.A.
Summary: RQF040</para>
      </listitem>
      <listitem>
	<para>VER019-VER021: versions on local disk</para>
      </listitem>
      <listitem xml:id="VER022" xreflabel="(VER022)">
	<para>VER022  (PVpub-AnalogClock.22.mcz, 15 June 2008, 7:34:42 pm)
intermediate version, no Q.A.
Summary: RQF041
</para>
      </listitem>
      <listitem xml:id="VER023" xreflabel="(VER023)">
	<para>VER023  (PVpub-AnalogClock.23.mcz, dd Mmm 2008, xx:xx:xx)
all methods classified=ok
SmallLint: bugs=0, possible bugs=8
SwaLint: LongMethods=35, Bad message=2, Guarding clauses=1, Debugging code left in methods=11, Uses (to:)do: instead of to:do:=3, Sends different super message=1
LOC-instance=882, LOC-class=81
tests passed: 2/2
review against K. Beck, SBPP, Chapter 7=ok
Summary: RQF040. RQF041</para>
      </listitem>
      <listitem>
	<para>VER024, VER025 skipped</para>
      </listitem>
      <listitem xml:id="VER026" xreflabel="(VER026)">
	<para>VER026  (PVpub-AnalogClock-pv.26.mcz, 2 January 2009, 12:42:26 am)
Lint=42: Bugs=0, Intention revealing=6, Miscellaneous=19, Possible bugs=8, Unnecessary code=9
LOC-instance=876, LOC-class=81
tests passed: 2/2
review against K. Beck, SBPP, Chapter 7: not done
Summary: added new bug reports
</para>
      </listitem>
      <listitem xml:id="VER027" xreflabel="(VER027)">
	<para>VER027  (PVpub-AnalogClock-pv.27.mcz, 13 March 2009, xx:xx:xx)
all methods classified=ok
Summary: RQF046, B018, B019, B022, B029</para>
      </listitem>
      <listitem xml:id="VER028" xreflabel="(VER028)">
	<para>VER028  (PVpub-AnalogClock-pv.28.mcz, 29 October 2009, :: pm)
Lint=xx: Bugs=x, Intention revealing=x, Miscellaneous=xx, Possible bugs=x, Unnecessary code=x
LOC-instance=xxx, LOC-class=xx
tests passed: x/y
review against K. Beck, SBPP, Chapter 7: xx
Summary: new requirements, RQF050, </para>
      </listitem>
      <listitem xml:id="VER029" xreflabel="(VER029)">
	<para>VER029  (PVpub-AnalogClock-pv.29.mcz, 26 August 2010, 4:48:39.212 pm)
Lint=xx: Bugs=x, Intention revealing=x, Miscellaneous=xx, Possible bugs=x, Unnecessary code=x
LOC-instance=xxx, LOC-class=xx
tests passed: x/y
review against K. Beck, SBPP, Chapter 7: xx
Summary: review of AnalogClockProjectManual</para>
      </listitem>
      <listitem xml:id="VER030" xreflabel="(VER030)">
	<para>VER030  (PVpub-AnalogClock-pv.30.mcz, 31 August 2010, 3:02:26.807 pm)
tests passed: 2/2
Summary: B021
</para>
      </listitem>
      <listitem xml:id="VER031" xreflabel="(VER031)">
	<para>VER031  (PVpub-AnalogClock-pv.31.mcz, 31 August 2010, 6:49:43.732 pm)
tests passed: 2/2
Summary: B023, B024, B025</para>
      </listitem>
      <listitem xml:id="VER032" xreflabel="(VER032)">
	<para>VER032  (PVpub-AnalogClock-pv.32.mcz, 31 August 2010, 8:23:19.314 pm)
tests passed: 2/2
Summary: RQF26
</para>
      </listitem>
      <listitem xml:id="VER033" xreflabel="(VER033)">
	<para>VER033  (PVpub-AnalogClock-pv.33.mcz, 2 September 2010, 8:32:36.612 pm )
tests passed: 2/2, code coverage: 56%
Summary: documentation
</para>
      </listitem>
      <listitem xml:id="VER034" xreflabel="(VER034)">
	<para>VER034  (PVpub-AnalogClock-pv.34.mcz, 4 September 2010, 12:45:33.173 am)
tests passed: 2/2, code coverage: 56%
Summary: B030-B032, B034, B036, B037</para>
      </listitem>
      <listitem xml:id="VER035" xreflabel="(VER035)">
	<para>VER035  (PVpub-AnalogClock-pv.35.mcz, 5 September 2010, 12:42:09.664 am)
tests passed: 4/4, code coverage: 72%
Summary: ConfigPanel separated from AnalogClock, added 2 tests.</para>
      </listitem>
      <listitem xml:id="VER036" xreflabel="(VER036)">
	<para>VER036  (PVpub-AnalogClock-pv.36.mcz, 9 September 2010, 12:29:43.718 am)
Summary: added project manual in DB-format</para>
      </listitem>
      <listitem xml:id="VER037" xreflabel="(VER037)">
	<para>VER037  (PVpub-AnalogClock-pv.37.mcz, 9 September 2010, 12:33:18.136 am)
Summary: added project manual in DB-format</para>
      </listitem>
      <listitem xml:id="VER038" xreflabel="(VER038)">
	<para>VER038  (PVpub-AnalogClock-pv.38.mcz)
Summary: upload erroneous</para>
      </listitem>
      <listitem xml:id="VER039" xreflabel="(VER039)">
	<para>VER039  (PVpub-AnalogClock-pv.39.mcz, 9 September 2010, 5:37:09.027 pm)
Summary: Experimental: DocBook for documentation
</para>
      </listitem>
      <listitem xml:id="VER040" xreflabel="(VER040)">
	<para>VER040  (PVpub-AnalogClock-pv.40.mcz, )
Summary: DocBook with indices, with xref's
</para>
      </listitem>
      <listitem xml:id="VER041" xreflabel="(VER041)">
	<para>VER041  (PVpub-AnalogClock-pv.41.mcz, )
Lint=xx: Bugs=x, Intention revealing=x, Miscellaneous=xx, Possible bugs=x, Unnecessary code=x
LOC-instance=xxx, LOC-class=xx
tests passed: x/y, code coverage: 
review against K. Beck, SBPP, Chapter 7: xx
Summary: 
</para>
      </listitem>
      <listitem xml:id="VER042" xreflabel="(VER042)">
	<para>VER042  (PVpub-AnalogClock-pv.42.mcz, 24 October 2010, 1:33:35.769 am)
Lint=xx: Bugs=x, Intention revealing=x, Miscellaneous=xx, Possible bugs=x, Unnecessary code=x
LOC-instance=xxx, LOC-class=xx
tests passed: 7/7, code coverage: 71%
review against K. Beck, SBPP, Chapter 7: xx
Summary: Modifications to supports some more tests.
</para>
      </listitem>
      <listitem xml:id="VER043" xreflabel="(VER043)">
	<para>VER043  (PVpub-AnalogClock-pv.43.mcz, 29 October 2010, 8:03:30.728 pm)
Lint=xx: Bugs=x, Intention revealing=x, Miscellaneous=xx, Possible bugs=x, Unnecessary code=x
LOC-instance=xxx, LOC-class=xx
tests passed: , code coverage: %
review against K. Beck, SBPP, Chapter 7: xx
Summary: Documentation now DocBook5->HTML->Texts.
</para>
      </listitem>
      <listitem xml:id="VER044" xreflabel="(VER044)">
	<para>VER044  (PVpub-AnalogClock-pv.44.mcz, 29 October 2010, 11:17:11.643 pm)
LOC-instance=995, LOC-class=79
Lint=xx: Bugs=x, Intention revealing=x, Miscellaneous=xx, Possible bugs=x, Unnecessary code=x
review against K. Beck, SBPP, Chapter 7: xx
Corresponding version of PVpubTests-AnalogClock: PVpubTests-AnalogClock-pv.18.mcz, )
Summary: Explanations how to document.
</para>
      </listitem>
      <listitem xml:id="VER045" xreflabel="(VER045)">
	<para>VER045  (PVpub-AnalogClock-pv.45.mcz, 1 November 2010, 12:19:44.392 pm)
Lint=xx: Bugs=x, Intention revealing=x, Miscellaneous=xx, Possible bugs=x, Unnecessary code=x
LOC-instance=xxx, LOC-class=xx
Corresponding version of PVpubTests-AnalogClock: PVpubTests-AnalogClock-pv.18.mcz)
tests passed: 14/14, code coverage: 71%
review against K. Beck, SBPP, Chapter 7: xx
Summary: Review of public interfaces (using PackageExplorer).
</para>
      </listitem>
      <listitem xml:id="VER046" xreflabel="(VER046)">
	<para>VER046  (PVpub-AnalogClock-pv.46.mcz, 1 November 2010, 11:04:21.644 pm)
Lint=xx: Bugs=x, Intention revealing=x, Miscellaneous=xx, Possible bugs=x, Unnecessary code=x
LOC-instance=xxx, LOC-class=xx
Corresponding version of PVpubTests-AnalogClock: PVpubTests-AnalogClock-pv.18.mcz)
tests passed: , code coverage: %
review against K. Beck, SBPP, Chapter 7: xx
Summary: Minor bugfix.
</para>
      </listitem>
      <listitem xml:id="VER047" xreflabel="(VER047)">
	<para>VER047  (PVpub-AnalogClock-pv.47.mcz, 6 November 2010, 12:35:13.353 am)
Lint=xx: Bugs=x, Intention revealing=x, Miscellaneous=xx, Possible bugs=x, Unnecessary code=x
LOC-instance=xxx, LOC-class=xx
Corresponding version of PVpubTests-AnalogClock: PVpubTests-AnalogClock-pv.18.mcz)
tests passed: 14/14, code coverage: %
review against K. Beck, SBPP, Chapter 7: xx
Summary: Naming spiralClock -> spiralClockOn.
</para>
      </listitem>
      <listitem xml:id="VER048" xreflabel="(VER048)">
	<para>VER048  (PVpub-AnalogClock-pv.48.mcz, 6 November 2010, 12:57:51.574 am)
Lint=xx: Bugs=x, Intention revealing=x, Miscellaneous=xx, Possible bugs=x, Unnecessary code=x
LOC-instance=xxx, LOC-class=xx
Corresponding version of PVpubTests-AnalogClock: PVpubTests-AnalogClock-pv.18.mcz)
tests passed: , code coverage: %
review against K. Beck, SBPP, Chapter 7: xx
Summary: corrected bad use of collect:-idiom.
</para>
      </listitem>
      <listitem xml:id="VER049" xreflabel="(VER049)">
	<para>VER049  (PVpub-AnalogClock-pv.49.mcz, 6 November 2010, 1:09:38.677 am)
Lint=xx: Bugs=x, Intention revealing=x, Miscellaneous=xx, Possible bugs=x, Unnecessary code=x
LOC-instance=xxx, LOC-class=xx
Corresponding version of PVpubTests-AnalogClock: PVpubTests-AnalogClock-pv.19.mcz)
Corresponding version of ical: ical-pmm.74.mcz)
tests passed: 14/14, code coverage: %
review against K. Beck, SBPP, Chapter 7: xx
Summary: pharo-ize: setNumericValue: does not exist there.
</para>
      </listitem>
      <listitem xml:id="VER050" xreflabel="(VER050)">
	<para>VER050  (PVpub-AnalogClock-pv.50.mcz, 6 November 2010, 11:58:45 pm)
Lint=xx: Bugs=x, Intention revealing=x, Miscellaneous=xx, Possible bugs=x, Unnecessary code=x
LOC-instance=xxx, LOC-class=xx
Corresponding version of PVpubTests-AnalogClock: PVpubTests-AnalogClock-pv.19.mcz)
Corresponding version of ical: ical-pmm.74.mcz)
tests passed: 14/14, code coverage: %
review against K. Beck, SBPP, Chapter 7: xx
Summary: pharo-ize: DialAndMechanics>>initialize to send self lock: true in order to have the embedding morph (AnalogClock) accept the yellow button from Pharo.
</para>
      </listitem>
<listitem xml:id="VER051" xreflabel="(VER051)">
	<para>VER051  (PVpub-AnalogClock-pv.51.mcz, 14 November 2010, 10:57:25.01 pm )
Lint=xx: Bugs=x, Intention revealing=x, Miscellaneous=xx, Possible bugs=x, Unnecessary code=x
LOC-instance=xxx, LOC-class=xx
Corresponding version of PVpubTests-AnalogClock: PVpubTests-AnalogClock-pv.19.mcz)
Corresponding version of ical: ical-pmm.74.mcz)
tests passed: 14/14, code coverage: 69%
review against K. Beck, SBPP, Chapter 7: xx
Summary: small refactorings after describing public methods.
</para>
      </listitem>
<listitem xml:id="VER052" xreflabel="(VER052)">
	<para>VER052  (PVpub-AnalogClock-pv.52.mcz, 9 January 2011, 3:10:13.538 pm)
Lint=xx: Bugs=x, Intention revealing=x, Miscellaneous=xx, Possible bugs=x, Unnecessary code=x
PVpub-AnalogClock, LOC-instance=1606, LOC-class=79
PVpub-Tests, LOC-instance=374, LOC-class=0
Corresponding version of PVpubTests-AnalogClock: PVpubTests-AnalogClock-pv.19.mcz
Corresponding version of ical: ical-pmm.74.mcz
tests passed: 14/14, code coverage: 70%
review against K. Beck, SBPP, Chapter 7: xx
Summary: Divided class DialAndMechanics into: Dial and Mechanics.
</para>
      </listitem>
<listitem xml:id="VER053" xreflabel="(VER053)">
	<para>VER053  (PVpub-AnalogClock-pv.53.mcz, 10 January 2011, 11:54:27.622 pm)
Corresponding version of PVpubTests-AnalogClock: PVpubTests-AnalogClock-pv.19.mcz
Corresponding version of ical: ical-pmm.74.mcz
PVpub-AnalogClock, LOC-instance=1613, LOC-class=79
PVpub-Tests, LOC-instance374, LOC-class=0
tests passed: 14/14 , code coverage: 70%
Lint=xx: Bugs=x, Intention revealing=x, Miscellaneous=xx, Possible bugs=x, Unnecessary code=x
review against K. Beck, SBPP, Chapter 7: xx
Summary: 1) Fixed bug in WatchHand>>drawOn:, 2) improved jitter measurement in WatchHand>>drawOn:.
</para>
      </listitem>
<listitem xml:id="VER054" xreflabel="(VER054)">
	<para>VER054  (PVpub-AnalogClock-pv.54.mcz, 15 January 2011, 3:51:36.983 pm)
Corresponding version of PVpubTests-AnalogClock: PVpubTests-AnalogClock-pv.20.mcz
Corresponding version of ical: ical-pmm.74.mcz
PVpub-AnalogClock, LOC-instance=1642, LOC-class=79
PVpub-Tests, LOC-instance=405, LOC-class=0
tests passed: 16/16, code coverage: 71%
Lint=xx: Bugs=x, Intention revealing=x, Miscellaneous=xx, Possible bugs=x, Unnecessary code=x
review against K. Beck, SBPP, Chapter 7: xx
Summary: Added AnalogClockAcceptanceTest>>testSecondsWatchhandJitter to ensure regular operation of seconds watchhand (without "hesitation").
</para>
      </listitem>
<listitem xml:id="VER055" xreflabel="(VER055)">
	<para>VER055</para>
      </listitem>
<listitem xml:id="VER056" xreflabel="(VER056)">
	<para>VER056</para>
      </listitem>
<listitem xml:id="VER057" xreflabel="(VER057)">
	<para>VER057  (PVpub-AnalogClock-pv.57.mcz, 3 June 2011, 11:20:02.986 pm)
(PVpubTests-AnalogClock-pv.24.mcz, 3 June 2011, 11:26:40.693 pm)
Version of Squeak: 4.2
Version of PVpubTests-AnalogClock: PVpubTests-AnalogClock-pv.20.mcz
Version of ical: ical-pmm.74.mcz
Version of SwaLint: ConfigurationOfSwaLint-tfel.5

PVpub-AnalogClock, LOC-instance=1620, LOC-class=93
PVpub-Tests, LOC-instance=287, LOC-class=8
tests passed=16/16, code coverage= 76%
methods left uncategorized=0
SwaLint on Pvpub-AnalogClock and PvpubTests-AnalogClock
SmallLint/Sends "questionable" message=(AnalogClock=2)
SmallLint/Utility methods=(Dial=1,Mechanics=1,SpiralCalendar=1)
SmallLint/Law of demeter=(Spiral=2,SpiralEvent=1,WatchHand=3)
review against K. Beck, SBPP, Chapter 7=ok
Summary: categorized methods, removed several Lint problems, checked dropping of user defined watchhand.
</para>
      </listitem>
<listitem xml:id="VER058" xreflabel="(VER058)">
	<para>VER058  (PVpub-AnalogClock-pv.58.mcz, 13 June 2011, 12:25:21.946 am)
(PVpubTests-AnalogClock-pv.24.mcz, )
	</para>
<para>
Version of Squeak: 4.2
Version of ical: ical-pmm.74.mcz
Version of SwaLint: ConfigurationOfSwaLint-tfel.5
</para>
<para>
Metrics:
PVpub-AnalogClock, LOC-instance=1669, LOC-class=95
PVpub-Tests, LOC-instance=287, LOC-class=8
tests passed=16/16, code coverage= 74%
methods left uncategorized=0
SwaLint on Pvpub-AnalogClock and PvpubTests-AnalogClock
SmallLint/Excessive numter of methods=(AnalogClock=1)
SmallLint/Sends "questionable" message=(AnalogClock=2)
SmallLint/Long methods=(ConfigPanel=1)
SmallLint/Utility methods=(Dial=1,SpiralCalendar=1)
SmallLint/Law of demeter=(Spiral=2,SpiralEvent=1,WatchHand=3)
review against K. Beck, SBPP, Chapter 7=ok
</para><para>
Quality goals:
min UTLOC/(FuncLOC+UTLOC)=20%
max UTLOC/(FuncLOC+UTLOC)=40%
real=14%
min unit test code coverage=80%
max unit test code coverage=90%
real=74%
min open bugs per kLOC=3
max open bugs per kLOC=10
real=3
min open lint problems per kLOC=5
max open lint problems per kLOC=10
real=6
min fan out of a class=1
max fan out of a class=10
real=AnalogClock(20)
</para>
<para>
Summary: rotationDue now called from WatchHand, not from AnalogClock.
</para>
      </listitem>
<listitem xml:id="VER059" xreflabel="(VER059)">
	<formalpara>
	  <title>Versions</title>
<para>VER059  (PVpub-AnalogClock-pv.59.mcz, 13 June 2011, 4:56:01.617 pm)
(PVpubTests-AnalogClock-pv.24.mcz, )
Version of Squeak: 4.2
Version of ical: ical-pmm.74.mcz
Version of SwaLint: ConfigurationOfSwaLint-tfel.5
</para></formalpara>
<formalpara>
<title>Metrics</title>
<para>
PVpub-AnalogClock, FuncLOC-instance=1661, FuncLOC-class=95, FuncLOC=1756
PVpub-Tests, UTLOC-instance=287, UTLOC-class=8, UTLOC=295
tests passed=16/16, code coverage= 74%
methods left uncategorized=0
SwaLint on Pvpub-AnalogClock and PvpubTests-AnalogClock
SmallLint/Sends "questionable" message=(AnalogClock=2)
SmallLint/Utility methods=(Dial=1,SpiralCalendar=1)
SmallLint/Law of demeter=(Spiral=2,SpiralEvent=1,WatchHand=3)
SmallLint N of open problems=10
review against K. Beck, SBPP, Chapter 7=ok
</para></formalpara>
<formalpara><title>Quality goals</title>
<para>
UTLOC/(FuncLOC+UTLOC)=(min=20%,max=40%,real=14%)
unit test code coverage=(min=80%,max=90%,real=74%)
open bugs per kLOC=(min=3,max=10,real=3)
open lint problems per kLOC=(min=5,max=10,real=5)
max fan out of a class=(min=1,max=10,real=20)
</para></formalpara>
<formalpara><title>
Summary</title>
<para>refactored AnalogClock (god class), new format for metrics.
</para></formalpara>
      </listitem>
<listitem xml:id="VER060" xreflabel="(VER060)">
<formalpara><title>Versions</title>
	<para>VER060  (PVpub-AnalogClock-pv.60.mcz, 14 June 2011, 10:18:26.221 pm)
(PVpubTests-AnalogClock-pv.24.mcz, )
Version of Squeak: 4.2
Version of ical: ical-pmm.74.mcz
Version of SwaLint: ConfigurationOfSwaLint-tfel.5
Version of Moose: 4_0-beta-4
	</para></formalpara>
<formalpara><title>Metrics</title><para>
Requirements=(RQF=54,RQN=9,RQP=2)
Requirements implemented=(RQF=38,RQN=7,RQP=1)
PVpub-AnalogClock, FuncLOC-instance=1661, FuncLOC-class=95, FuncLOC=1756
PVpub-Tests, UTLOC-instance=287, UTLOC-class=8, UTLOC=295
tests passed=16/16, code coverage= 74%
methods left uncategorized=0
SwaLint on Pvpub-AnalogClock and PvpubTests-AnalogClock
SmallLint/Sends "questionable" message=(AnalogClock=2)
SmallLint/Utility methods=(Dial=1,SpiralCalendar=1)
SmallLint/Law of demeter=(Spiral=2,SpiralEvent=1,WatchHand=3)
SmallLint N of open problems=10
review against K. Beck, SBPP, Chapter 7=ok
</para></formalpara>
<formalpara><title>Quality goals</title><para>
Requirements implemented/total requirements=(min=70%,max=90%,real=71%)
UTLOC/(FuncLOC+UTLOC)=(min=20%,max=40%,real=14%)
unit test code coverage=(min=80%,max=90%,real=74%)
open bugs per kLOC=(min=3,max=10,real=3)
open lint problems per kLOC=(min=5,max=10,real=5)
max fan out of a class=(min=1,max=10,real=20)
</para></formalpara>
<formalpara><title>Summary</title><para>Project manual added in class comment of AnalogClock.
</para></formalpara>
      </listitem>
<listitem xml:id="VER061" xreflabel="(VER061)">
<formalpara><title>Versions</title>
	<para>VER061 (PVpub-AnalogClock-pv.61.mcz, 25 June 2011, 11:43:17.108 pm)
(PVpubTests-AnalogClock-pv.25.mcz, )
Version of Squeak: 4.2
Version of ical: ical-pmm.74.mcz
Version of SwaLint: ConfigurationOfSwaLint-tfel.5
Version of Moose: 4_5
	</para></formalpara>
<formalpara><title>Metrics</title><para>
Requirements=(RQF=54,RQN=9,RQP=2)
Requirements implemented=(RQF=38,RQN=7,RQP=1)
Open bugs=(BugCount=7)
PVpub-AnalogClock, FuncLOC-instance=1110, FuncLOC-class=131, FuncLOC=1241
PVpub-Tests, UTLOC-instance=287, UTLOC-class=8, UTLOC=295
tests passed=16/16, code coverage= 78%
methods left uncategorized=0
SwaLint on Pvpub-AnalogClock and PvpubTests-AnalogClock
SmallLint/Sends "questionable" message=(AnalogClock=2)
SmallLint N of open problems=(SmallLintCount=2)
review against K. Beck, SBPP, Chapter 7=ok
</para></formalpara>
<formalpara><title>Quality goals</title><para>
Requirements implemented/total requirements=(min=70%,max=90%,real=71%)
UTLOC/(FuncLOC+UTLOC)=(min=20%,max=40%,real=19%)
unit test code coverage=(min=80%,max=90%,real=78%)
BugCount*1000/(FuncLOC+UTLOC)=(min=3,max=10,real=5)
SmallLintCount*1000/(FuncLOC+UTLOC)=(min=5,max=10,real=1)
max fan out of a class=(min=1,max=10,real=18)
</para></formalpara>
<formalpara><title>Summary</title><para>Reduced SmallLint N of open problems to 2.
</para></formalpara>
      </listitem>
<listitem xml:id="VER062" xreflabel="(VER062)">
<formalpara><title>Versions</title>
	<para>VER062 (PVpub-AnalogClock-pv.62.mcz, 10 July 2011, 12:08:21.892 am)
(PVpubTests-AnalogClock-pv.25.mcz, 2011-06-25 23:39:18)
Version of Squeak: 4.2
Version of ical: ical-pmm.74.mcz
Version of SwaLint: ConfigurationOfSwaLint-tfel.5
Version of Moose: 4_5
	</para></formalpara>
<formalpara><title>Metrics</title><para>
Requirements=(RQF=54,RQN=9,RQP=2)
Requirements implemented=(RQF=38,RQN=7,RQP=1)
Open bugs=(BugCount=7)
PVpub-AnalogClock, FuncLOC-instance=1110, FuncLOC-class=131, FuncLOC=1241
PVpub-Tests, UTLOC-instance=287, UTLOC-class=8, UTLOC=295
tests passed=16/16, code coverage= 78%
methods left uncategorized=0
SwaLint on Pvpub-AnalogClock and PvpubTests-AnalogClock
SmallLint/Sends "questionable" message=(AnalogClock=2)
SmallLint N of open problems=(SmallLintCount=2)
review against K. Beck, SBPP, Chapter 7=ok
</para></formalpara>
<formalpara><title>Quality goals</title><para>
Requirements implemented/total requirements=(min=70%,max=90%,real=71%)
UTLOC/(FuncLOC+UTLOC)=(min=20%,max=40%,real=19%)
unit test code coverage=(min=80%,max=90%,real=78%)
BugCount*1000/(FuncLOC+UTLOC)=(min=3,max=10,real=5)
SmallLintCount*1000/(FuncLOC+UTLOC)=(min=5,max=10,real=2)
max fan out of a class=(min=1,max=10,real=18)
</para></formalpara>
<formalpara><title>Summary</title><para>Class comments following Squeak template for all classes. Plus: responsibilities and collaborators.
</para></formalpara>
      </listitem>
<listitem xml:id="VER063" xreflabel="(VER063)">
<formalpara><title>Versions</title>
	<para>VER063 (PVpub-AnalogClock-pv.63.mcz, 31 July 2011, 2:46:25.842 pm)
(PVpubTests-AnalogClock-pv.25.mcz, )
Version of Squeak: 4.2-10966
Version of ical: ical-pmm.74.mcz
Version of SwaLint: ConfigurationOfSwaLint-tfel.5
Version of Moose: 4_5
	</para></formalpara>
<formalpara><title>Metrics</title><para>
Requirements=(RQF=58,RQN=9,RQP=2)
Requirements implemented=(RQF=38,RQN=7,RQP=1)
Open bugs=(BugCount=8)
PVpub-AnalogClock, FuncLOC-instance=1140, FuncLOC-class=131, FuncLOC=1271
PVpub-Tests, UTLOC-instance=287, UTLOC-class=8, UTLOC=295
tests passed=16/16, code coverage= 76%
methods left uncategorized=0
SwaLint on PVpub-AnalogClock and PVpubTests-AnalogClock
SmallLint/Sends "questionable" message=(AnalogClock=2)
SmallLint/Utility methods=(Spiral=1)
SmallLint N of open problems=(SmallLintCount=3)
Moose/InvokedClasses in Smalltalk:xx=(AnalogClock=17)
review against K. Beck, SBPP, Chapter 7=ok
</para></formalpara>
<formalpara><title>Quality goals</title><para>
Requirements implemented/total requirements=(min=70%,max=90%,real=66%)
UTLOC/(FuncLOC+UTLOC)=(min=20%,max=40%,real=19%)
unit test code coverage=(min=80%,max=90%,real=76%)
BugCount*1000/(FuncLOC+UTLOC)=(min=3,max=10,real=8)
SmallLintCount*1000/(FuncLOC+UTLOC)=(min=5,max=10,real=2)
max fan out of a class=(min=1,max=10,real=17)
</para></formalpara>
<formalpara><title>Summary</title><para>Bugfix BUG047, BUG048.
</para></formalpara>
      </listitem>
<listitem xml:id="VER064" xreflabel="(VER064)">
<formalpara><title>Versions</title>
	<para>VER064 
(PVpub-AnalogClock-pv.64.mcz, 12 February 2012, 9:53:02.744 pm)
(PVpubTests-AnalogClock-pv.27.mcz, 12 February 2012, 9:55:27.293 pm)
Version of Squeak: 4.2-10966
Version of ical: ical-pmm.74.mcz
Version of SwaLint: ConfigurationOfSwaLint-tfel.5
Version of Moose: 4_6_Development
	</para></formalpara>
<formalpara><title>Metrics</title><para>
Requirements=(RQF=63,RQN=10,RQP=4). Sum=77
Requirements implemented=(RQF=38,RQN=6,RQP=1). Sum=45
Open bugs=(BugCount=52,BugsClosed=43,BugsOpen=9)
PVpub-AnalogClock, FuncLOC-instance=1333, FuncLOC-class=138, FuncLOC=1471
PVpub-Tests, UTLOC-instance=296, UTLOC-class=8, UTLOC=304
tests passed=16/16, code coverage= 76%
methods left uncategorized=0
SwaLint on PVpub-AnalogClock and PVpubTests-AnalogClock
SmallLint/Sends "questionable" message=(AnalogClock=2)
SmallLint N of open problems=(SmallLintCount=2)
Moose/max(InvokedClasses)=(AnalogClockViewController=19)
Moose/max(InvokedClasses)=(AnalogClockViewController=19)
Moose/max(InvokedClasses)-inside package for AnalogClockViewController=10
Moose/max(InvokedClasses)-outside package for AnalogClockViewController=9
review against K. Beck, SBPP, Chapter 7=not done
</para></formalpara>
<formalpara><title>Quality goals</title><para>
Requirements implemented/total requirements=(min=70%,max=90%,real=58%)
UTLOC/(FuncLOC+UTLOC)=(min=20%,max=40%,real=17%)
unit test code coverage=(min=80%,max=90%,real=78%)
BugsOpen*1000/(FuncLOC+UTLOC)=(min=3,max=10,real=5)
SmallLintCount*1000/(FuncLOC+UTLOC)=(min=5,max=10,real=1)
max fan out of a class=(min=1,max=10,real=19)
</para></formalpara>
<formalpara><title>Summary</title><para>Major refactoring, AnalogClock has given away responsibilities to AnalogClockViewController. Review of functional requirements.
</para></formalpara>
      </listitem>
<listitem xml:id="VER065" xreflabel="(VER065)">
<formalpara><title>Versions</title>
	<para>VER065 
(PVpub-AnalogClock-pv.01.mcz, 19 January 2013, 9:34:56.421 pm)
(PVpubTests-AnalogClock-pv.01.mcz, 19 January 2013, 9:44:53.26 pm)
Version of Squeak: 4.3
Version of ical: /smalltalk/sq4.2-10966/Contents/Resources/ICal-pv.75.mcz
(This is a local copy of ical-pmm.74.mcz which is not available on www.squeaksource.com.)
Version of SwaLint: ConfigurationOfSwaLint-tfel.11 >>>>> 2013-01-20pv not able to load
Version of Moose: 4_6_Development
	</para></formalpara>
<formalpara><title>Metrics</title><para>
Requirements=(RQF=63,RQN=10,RQP=4). Sum=77
Requirements implemented=(RQF=38,RQN=6,RQP=1). Sum=45
Open bugs=(BugCount=52,BugsClosed=43,BugsOpen=9)
PVpub-AnalogClock, FuncLOC-instance=1333, FuncLOC-class=138, FuncLOC=1471
PVpub-Tests, UTLOC-instance=296, UTLOC-class=8, UTLOC=304
tests passed=16/16, code coverage= 76%
methods left uncategorized=0
SwaLint on PVpub-AnalogClock and PVpubTests-AnalogClock
SmallLint/Sends "questionable" message=(AnalogClock=2)
SmallLint N of open problems=(SmallLintCount=2)
Moose/max(InvokedClasses)=(AnalogClockViewController=19)
Moose/max(InvokedClasses)=(AnalogClockViewController=19)
Moose/max(InvokedClasses)-inside package for AnalogClockViewController=10
Moose/max(InvokedClasses)-outside package for AnalogClockViewController=9
review against K. Beck, SBPP, Chapter 7=not done
</para></formalpara>
<formalpara><title>Quality goals</title><para>
Requirements implemented/total requirements=(min=70%,max=90%,real=xx%)
UTLOC/(FuncLOC+UTLOC)=(min=20%,max=40%,real=xx%)
unit test code coverage=(min=80%,max=90%,real=xx%)
BugsOpen*1000/(FuncLOC+UTLOC)=(min=3,max=10,real=xx)
SmallLintCount*1000/(FuncLOC+UTLOC)=(min=5,max=10,real=xx)
max fan out of a class=(min=1,max=10,real=xx)
</para></formalpara>
<formalpara><title>Summary</title><para>Migration from squeaksource.com to ss3.gemstone.com/ss.
</para></formalpara>
      </listitem>
<listitem xml:id="VER066" xreflabel="(VER066)">
<formalpara><title>Versions</title>
	<para>VER066 
(PVpub-AnalogClock-pv.02.mcz, 23 February 2013, 12:34:19.075 am)
(PVpubTests-AnalogClock-pv.01.mcz, 19 January 2013,xxxxxxxxxxxxxxxxxxx)
Version of Squeak: 4.3
Version of ical: ICal-YanniChiu.87
Version of SwaLint: ConfigurationOfSwaLint-tfel.5
Version of Moose: 4_6_Development
	</para></formalpara>
<formalpara><title>Metrics</title><para>
Requirements=(RQF=63,RQN=10,RQP=4). Sum=77
Requirements implemented=(RQF=38,RQN=6,RQP=1). Sum=45
Open bugs=(BugCount=52,BugsClosed=43,BugsOpen=9)
PVpub-AnalogClock, FuncLOC-instance=1333, FuncLOC-class=138, FuncLOC=1471
PVpub-Tests, UTLOC-instance=296, UTLOC-class=8, UTLOC=304
tests passed=16/16, code coverage= 76%
methods left uncategorized=0
SwaLint on PVpub-AnalogClock and PVpubTests-AnalogClock
SmallLint/Sends "questionable" message=(AnalogClock=2)
SmallLint N of open problems=(SmallLintCount=2)
Moose/max(InvokedClasses)=(AnalogClockViewController=19)
Moose/max(InvokedClasses)=(AnalogClockViewController=19)
Moose/max(InvokedClasses)-inside package for AnalogClockViewController=10
Moose/max(InvokedClasses)-outside package for AnalogClockViewController=9
review against K. Beck, SBPP, Chapter 7=not done
</para></formalpara>
<formalpara><title>Quality goals</title><para>
Requirements implemented/total requirements=(min=70%,max=90%,real=xx%)
UTLOC/(FuncLOC+UTLOC)=(min=20%,max=40%,real=xx%)
unit test code coverage=(min=80%,max=90%,real=xx%)
BugsOpen*1000/(FuncLOC+UTLOC)=(min=3,max=10,real=x)
SmallLintCount*1000/(FuncLOC+UTLOC)=(min=5,max=10,real=x)
max fan out of a class=(min=1,max=10,real=xx)
</para></formalpara>
<formalpara><title>Summary</title><para>Repaired loading of ical. Updated AnalogClockDocumentation.
</para></formalpara>
      </listitem>
<listitem xml:id="VER067" xreflabel="(VER067)">
<formalpara><title>Versions</title>
	<para>VER067 
(PVpub-AnalogClock-pv.03.mcz, 24 February 2013, 9:37:17.063 pm)
(PVpubTests-AnalogClock-pv.02.mcz, 24 February 2013, 9:31:33.378 pm)
Version of Squeak: 4.3
Version of ical: ConfigurationOfICal project version: '1.1'
Version of SwaLint: ConfigurationOfSwaLint project version: '2.0.0', ignoring warning about dependencies of SLGreasePlugIn and SLSlimePlugIn
Version of Moose: 4_6_Development
	</para></formalpara>
<formalpara><title>Metrics</title><para>
Requirements=(RQF=63,RQN=10,RQP=4). Sum=77
Requirements implemented=(RQF=38,RQN=6,RQP=1). Sum=45
Open bugs=(BugCount=52,BugsClosed=43,BugsOpen=9)
PVpub-AnalogClock, FuncLOC-instance=1338, FuncLOC-class=138, FuncLOC=1476
PVpub-Tests, UTLOC-instance=296, UTLOC-class=8, UTLOC=304
TestRunner=(tests passed=16/16, code coverage= 81%)
SystemBrowser=(methods left uncategorized=0)
SwaLint on PVpub-AnalogClock and PVpubTests-AnalogClock
SmallLint/Sends "questionable" message=(AnalogClock=2)
SmallLint/Variable referenced in only one method and always assigned first=(AnalogClockTest=1)
SmallLint/Messages sent but not implemented=(WatchHand=2)
SmallLint N of open problems=(SmallLintCount=5)
Moose/max(InvokedClasses)=(AnalogClockViewController=xx)
Moose/max(InvokedClasses)=(AnalogClockViewController=xx)
Moose/max(InvokedClasses)-inside package for AnalogClockViewController=xx
Moose/max(InvokedClasses)-outside package for AnalogClockViewController=x
review against K. Beck, SBPP, Chapter 7=not done
</para></formalpara>
<formalpara><title>Quality goals</title><para>
Requirements implemented/total requirements=(min=70%,max=90%,real=xx%)
UTLOC/(FuncLOC+UTLOC)=(min=20%,max=40%,real=17%)
unit test code coverage=(min=80%,max=90%,real=81%)
BugsOpen*1000/(FuncLOC+UTLOC)=(min=3,max=10,real=5)
SmallLintCount*1000/(FuncLOC+UTLOC)=(min=5,max=10,real=3)
max fan out of a class=(min=1,max=10,real=xx)
</para></formalpara>
<formalpara><title>Summary</title><para>Unit tests and SmallLint run again.
</para></formalpara>
      </listitem>
<listitem xml:id="VER068" xreflabel="(VER068)">
<formalpara><title>Versions</title>
	<para>VER068
(PVpub-AnalogClock-pv.4.mcz, 26 February 2013, 10:39:15.693 am)
(PVpubTests-AnalogClock-pv.3.mcz, 26 February 2013, 10:37:38.183 am)
Version of Squeak: 4.3
Version of ical: ConfigurationOfICal project version: '1.1'
Version of SwaLint: ConfigurationOfSwaLint project version: '2.0.0', ignoring warning about dependencies of SLGreasePlugIn and SLSlimePlugIn
Version of Moose: 4_6_Development
	</para></formalpara>
<formalpara><title>Metrics</title><para>
Requirements=(RQF=63,RQN=10,RQP=4). Sum=77
Requirements implemented=(RQF=38,RQN=6,RQP=1). Sum=45
Open bugs=(BugCount=52,BugsClosed=43,BugsOpen=9)
PVpub-AnalogClock, FuncLOC-instance=1338, FuncLOC-class=138, FuncLOC=1476
PVpub-Tests, UTLOC-instance=298, UTLOC-class=8, UTLOC=306
TestRunner=(tests passed=15/16, code coverage= 81%)
SystemBrowser=(methods left uncategorized=0)
SwaLint on PVpub-AnalogClock and PVpubTests-AnalogClock
SmallLint/Sends "questionable" message=(AnalogClock=2)
SmallLint/Law of demeter=(Tests-AnalogClock=2)
SmallLint/Variable referenced in only one method and always assigned first=(Test-AnalogClock)
SmallLint/Messages sent but not implemented=(WatchHand=2)
SmallLint N of open problems=(SmallLintCount=7)
Moose/max(InvokedClasses)=(AnalogClockViewController=xx)
Moose/max(InvokedClasses)=(AnalogClockViewController=xx)
Moose/max(InvokedClasses)-inside package for AnalogClockViewController=xx
Moose/max(InvokedClasses)-outside package for AnalogClockViewController=x
review against K. Beck, SBPP, Chapter 7=not done
</para></formalpara>
<formalpara><title>Quality goals</title><para>
Requirements implemented/total requirements=(min=70%,max=90%,real=xx%)
UTLOC/(FuncLOC+UTLOC)=(min=20%,max=40%,real=17%)
unit test code coverage=(min=80%,max=90%,real=81%)
BugsOpen*1000/(FuncLOC+UTLOC)=(min=3,max=10,real=5)
SmallLintCount*1000/(FuncLOC+UTLOC)=(min=5,max=10,real=4)
max fan out of a class=(min=1,max=10,real=xx)
</para></formalpara>
<formalpara><title>Summary</title><para>1. Changed UpdatedSimpleButtonMorph to SimpleButtonMorph (Pharo does not know UpdatedSimpleButtonMorph). 2. Left out timeout: (Pharo does not know TestCase>>timeout:) Used method annotation in Pharo. Used (SystemVersion current version beginsWith: 'Squeak') ifTrue: [self timeout: 15].
</para></formalpara>
      </listitem>
<listitem xml:id="VER069" xreflabel="(VER069)">
<formalpara><title>Versions</title>
	<para>VER069
(PVpub-AnalogClock-pv.5.mcz, 26 February 2013, 10:50:55.403 am)
(PVpubTests-AnalogClock-pv.3.mcz, 26 February 2013, 10:37:38.183 am)
Version of Squeak: 4.3
Version of ical: ConfigurationOfICal project version: '1.1'
Version of SwaLint: ConfigurationOfSwaLint project version: '2.0.0', ignoring warning about dependencies of SLGreasePlugIn and SLSlimePlugIn
Version of Moose: 4_6_Development
	</para></formalpara>
<formalpara><title>Metrics</title><para>
Requirements=(RQF=63,RQN=10,RQP=4). Sum=77
Requirements implemented=(RQF=38,RQN=6,RQP=1). Sum=45
Open bugs=(BugCount=52,BugsClosed=43,BugsOpen=9)
PVpub-AnalogClock, FuncLOC-instance=1338, FuncLOC-class=138, FuncLOC=1476
PVpub-Tests, UTLOC-instance=298, UTLOC-class=8, UTLOC=306
TestRunner=(tests passed=15/16, code coverage= 81%)
SystemBrowser=(methods left uncategorized=0)
SwaLint on PVpub-AnalogClock and PVpubTests-AnalogClock
SmallLint/Sends "questionable" message=(AnalogClock=2)
SmallLint/Law of demeter=(Tests-AnalogClock=2)
SmallLint/Variable referenced in only one method and always assigned first=(Test-AnalogClock)
SmallLint/Messages sent but not implemented=(WatchHand=2)
SmallLint N of open problems=(SmallLintCount=7)
Moose/max(InvokedClasses)=(AnalogClockViewController=xx)
Moose/max(InvokedClasses)=(AnalogClockViewController=xx)
Moose/max(InvokedClasses)-inside package for AnalogClockViewController=xx
Moose/max(InvokedClasses)-outside package for AnalogClockViewController=x
review against K. Beck, SBPP, Chapter 7=not done
</para></formalpara>
<formalpara><title>Quality goals</title><para>
Requirements implemented/total requirements=(min=70%,max=90%,real=xx%)
UTLOC/(FuncLOC+UTLOC)=(min=20%,max=40%,real=17%)
unit test code coverage=(min=80%,max=90%,real=81%)
BugsOpen*1000/(FuncLOC+UTLOC)=(min=3,max=10,real=5)
SmallLintCount*1000/(FuncLOC+UTLOC)=(min=5,max=10,real=4)
max fan out of a class=(min=1,max=10,real=xx)
</para></formalpara>
<formalpara><title>Summary</title><para>Changed UpdatedSimpleButtonMorph to SimpleButtonMorph (Pharo does not know UpdatedSimpleButtonMorph).
</para></formalpara>
      </listitem>
<listitem xml:id="VER070" xreflabel="(VER070)">
<formalpara><title>Versions</title>
	<para>VER070
(PVpub-AnalogClock-pv.06.mcz, 17 March 2013, xxxxxxxxxxxxxxxxxxx)
(PVpubTests-AnalogClock-pv.04.mcz, 17 March 2013,xxxxxxxxxxxxxxxxxxx)
Version of Squeak: 4.3
Version of ical: ConfigurationOfICal project version: '1.1'
Version of SwaLint: ConfigurationOfSwaLint project version: '2.0.0', ignoring warning about dependencies of SLGreasePlugIn and SLSlimePlugIn
Version of Moose: 4_7_Development
	</para></formalpara>
<formalpara><title>Metrics</title><para>
Requirements=(RQF=63,RQN=10,RQP=4). Sum=77
Requirements implemented=(RQF=38,RQN=6,RQP=1). Sum=45
Open bugs=(BugCount=52,BugsClosed=43,BugsOpen=9)
PVpub-AnalogClock, FuncLOC-instance=1338, FuncLOC-class=138, FuncLOC=1476
PVpub-Tests, UTLOC-instance=296, UTLOC-class=8, UTLOC=304
TestRunner=(tests passed=16/16, code coverage= 76%)
SystemBrowser=(methods left uncategorized=0)
SwaLint on PVpub-AnalogClock and PVpubTests-AnalogClock
SmallLint/Sends "questionable" message=(AnalogClock=2)
SmallLint/Variable referenced in only one method and always assigned first=(AnalogClockTest=1)
SmallLint/Messages sent but not implemented=(WatchHand=2)
SmallLint N of open problems=(SmallLintCount=5)
Moose/max(InvokedClasses)=(AnalogClockViewController=xx)
Moose/max(InvokedClasses)=(AnalogClockViewController=xx)
Moose/max(InvokedClasses)-inside package for AnalogClockViewController=xx
Moose/max(InvokedClasses)-outside package for AnalogClockViewController=x
review against K. Beck, SBPP, Chapter 7=not done
</para></formalpara>
<formalpara><title>Quality goals</title><para>
Requirements implemented/total requirements=(min=70%,max=90%,real=xx%)
UTLOC/(FuncLOC+UTLOC)=(min=20%,max=40%,real=17%)
unit test code coverage=(min=80%,max=90%,real=xx%)
BugsOpen*1000/(FuncLOC+UTLOC)=(min=3,max=10,real=x)
SmallLintCount*1000/(FuncLOC+UTLOC)=(min=5,max=10,real=x)
max fan out of a class=(min=1,max=10,real=xx)
</para></formalpara>
<formalpara><title>Summary</title><para>method String>>withoutTrailingBlanks known in Squeak is String>>trimRight in Pharo.
</para></formalpara>
      </listitem>
<listitem xml:id="VER071" xreflabel="(VER071)">
<formalpara><title>Versions</title>
	<para>VER071
(PVpub-AnalogClock-pv.06.mcz, 17 March 2013, xxxxxxxxxxxxxxxxxxx)
(PVpubTests-AnalogClock-pv.04.mcz, 17 March 2013,xxxxxxxxxxxxxxxxxxx)
Version of Squeak: 4.3
Version of ical: ConfigurationOfICal project version: '1.1'
Version of SwaLint: ConfigurationOfSwaLint project version: '2.0.0', ignoring warning about dependencies of SLGreasePlugIn and SLSlimePlugIn
Version of Moose: 4_7_Development
	</para></formalpara>
<formalpara><title>Metrics</title><para>
Requirements=(RQF=63,RQN=10,RQP=4). Sum=77
Requirements implemented=(RQF=38,RQN=6,RQP=1). Sum=45
Open bugs=(BugCount=52,BugsClosed=43,BugsOpen=9)
PVpub-AnalogClock, FuncLOC-instance=1338, FuncLOC-class=138, FuncLOC=1476
PVpub-Tests, UTLOC-instance=296, UTLOC-class=8, UTLOC=304
TestRunner=(tests passed=16/16, code coverage= 76%)
SystemBrowser=(methods left uncategorized=0)
SwaLint on PVpub-AnalogClock and PVpubTests-AnalogClock
SmallLint/Sends "questionable" message=(AnalogClock=2)
SmallLint/Variable referenced in only one method and always assigned first=(AnalogClockTest=1)
SmallLint/Messages sent but not implemented=(WatchHand=2)
SmallLint N of open problems=(SmallLintCount=5)
Moose/max(InvokedClasses)=(AnalogClockViewController=xx)
Moose/max(InvokedClasses)=(AnalogClockViewController=xx)
Moose/max(InvokedClasses)-inside package for AnalogClockViewController=xx
Moose/max(InvokedClasses)-outside package for AnalogClockViewController=x
review against K. Beck, SBPP, Chapter 7=not done
</para></formalpara>
<formalpara><title>Quality goals</title><para>
Requirements implemented/total requirements=(min=70%,max=90%,real=xx%)
UTLOC/(FuncLOC+UTLOC)=(min=20%,max=40%,real=17%)
unit test code coverage=(min=80%,max=90%,real=xx%)
BugsOpen*1000/(FuncLOC+UTLOC)=(min=3,max=10,real=x)
SmallLintCount*1000/(FuncLOC+UTLOC)=(min=5,max=10,real=x)
max fan out of a class=(min=1,max=10,real=xx)
</para></formalpara>
<formalpara><title>Summary</title><para>...
</para></formalpara>
      </listitem>
    </itemizedlist>
  </section>
  <section>
    <title>Bugs</title>
    <para>Find in parenthesis at the start of the bug description the version number where the bug has been discovered. Find in parenthesis at the end of the bug description the version number where the bug has been fixed or confirmed to be fixed. "R2" for instance is for the monticello package with the suffix "*.2". Sometimes there is an explanation about the fix.</para>
    <itemizedlist spacing="compact">
      <listitem>
	<para><indexterm type="BUG"><primary>BUG001</primary></indexterm>BUG001 while dragging with the right mouse button, watchhands loose their position and position does not get recovered.<literal>(VER003,VER012)</literal></para>
      </listitem>
      <listitem>
	<para><indexterm type="BUG"><primary>BUG002</primary></indexterm>BUG002 two clocks overlap partially: the watchhands of the clock with higher z-values are drawn partially when clocks with lower z-values are updated.<literal>(VER012)</literal></para>
      </listitem>
      <listitem>
	<para>BUG003 CircleMorph>>rotationCenter can have a 'zero divide' when resizing with bottom right halo (see CircleMorph>>rotationCenter where zero divide is created) (VER007,VER012)</para>
      </listitem>
      <listitem>
	<para>BUG004 when clock radius gets changed: resize font of dial figures (VER006,VER012)</para>
      </listitem>
      <listitem>
	<para>BUG005 figures on dial should not be editable. Solution: FigureMorph>>handlesMouseDown. (VER012)</para>
      </listitem>
      <listitem xml:id="BUG006" xreflabel="(BUG006)">
	<para>BUG006 while the second watchhand is moved, the figures of the dial loose some pixel and gain them again later on.
Hint: Preferences/debugShowDamage set to true shows that pixel loss is along the sides of the damaged region. 
Root cause: fullBounds of TextMorph were computed (in syncMidnightShape) to floating values. (VER012)</para>
      </listitem>
      <listitem>
	<para>BUG007 when it is 12:00 make sure that three watchhands one on top of the other are clearly recognized. (VER012)</para>
      </listitem>
      <listitem>
	<para>BUG008 Let the minute watchhand pass to the full hour first, then let the spiral follow (VER014)</para>
      </listitem>
      <listitem>
	<para>BUG009 If AnalogClock is shrinked and event does not fit any more, event is displaced (VER010,VER013)</para>
      </listitem>
      <listitem>
	<para>BUG010 (VER007) When AnalogClock is moved strange rests of events are shown sometimes in the Morph's frame' (VER010)</para>
      </listitem>
      <listitem>
	<para>BUG011 (VER007) past events remain (VER013)</para>
      </listitem>
      <listitem>
	<para>BUG012 (VER007) when I try to drag the AnalogClock the mouse hovering over a figure of the dial: nothing will happen (VER012) </para>
      </listitem>
      <listitem>
	<para>BUG013 (VER007) when I change the size to smaller: events may stay too big and outside (VER010)</para>
      </listitem>
      <listitem>
	<para>BUG014 (VER008) a weird RectangleMorph can be seen at 0@0 when dragging and dropping AnalogClock several times near 0@0. (VER010)</para>
      </listitem>
      <listitem>
	<para>BUG015 (VER011) DragNDrop one AnalogClock over another: it will disappear (VER013)</para>
      </listitem>
      <listitem>
	<para>BUG015A (VER011) SpiraClock-events are shown 1 minute too early. This is "repaired" when second watchhand reaches zero. (VER013)</para>
      </listitem>
      <listitem>
	<para>BUG016 (VER012) drawNdrag 10-20 times. Starting from HandMorph there is a problem with Dial drawOn: (VER013)</para>
      </listitem>
      <listitem>
	<para>BUG017 (VER012) set an event that is shown on the clock. Resize until the event disappears. Resize to larger. Event does not appear again. (VER013)</para>
      </listitem>
      <listitem>
	<para>BUG018 (VER014) configuration panel is placed with a fix offset with respect to AnalogClock. If this doesn't fit on the screen choose another position which is more appropriate. (Check the aptitude of the four corners of the bounding box of AnalogClock. If none of them is good, take the center.) (VER027)</para>
      </listitem>
      <listitem>
	<para>BUG019 (VER014) configuration panel should get a name like 'AnalogClock-Panel'. (VER027)</para>
      </listitem>
      <listitem>
	<para>BUG020 (VER015) hypertext class comments vanish after exporting to and reimporting from Monticello. For instance this one to AnalogClock. Solution: not observed anymore on 08/26/2010 (VER029)</para>
      </listitem>
      <listitem>
	<para>BUG021 (VER015) when calendar events are dropped they are displayed only when passing the next full minute. (see also BUG028) (VER030)</para>
      </listitem>
      <listitem>
	<para>BUG022 stop second watchhand when incrementing or decrementing (VER027)</para>
      </listitem>
      <listitem>
	<para>BUG023 prevent user to drag and move "Dial" with right mouse button (VER031)</para>
      </listitem>
      <listitem>
	<para>BUG024 prevent user from duplicating watchhands (wantsHaloFromClick) (VER031)</para>
      </listitem>
      <listitem>
	<para>BUG025 do not display the date field when it gets outside of the clock (VER031)</para>
      </listitem>
      <listitem>
	<para>BUG026 treat ConfigPanel correctly after duplicating AnalogClocks. (VER032)</para>
      </listitem>
      <listitem>
	<para>BUG027 when deploying as standalone to a linux-system the window title in linux is squeak.image (instead of AnalogClock). Solution: rename the image from squeak.image to AnalogClock.image. Then, at least, the window title will be "AnalogClock.image". (VER033)</para>
      </listitem>
      <listitem>
	<para>BUG028 when running AnalogClockTest>>testDisplayCalendarEvent the second event is displayed only when the seconds watchhands passes the zero position - otherwise it is shown as 'abc' in the upper left corner of the display (VER032)</para>
      </listitem>
      <listitem>
	<para>BUG029 ConfigPanel shows: inc-start-reset-dec-inc. There is something wrong here. (VER027)</para>
      </listitem>
      <listitem>
	<para>BUG030 minutes on AnalogClock = minutes on Linux-system + 1 (VER034)</para>
      </listitem>
      <listitem>
	<para>BUG031 drop an ical-event on the spiraclock. Wait until you see it. Resize the clock. Most of the times a residual of the event at the old position remains. (VER034)</para>
      </listitem>
      <listitem>
	<para>BUG032 drop an ical-event on the spiraclock. Wait until you see it. Duplicate the clock. Sometimes the event is duplicated as well, sometimes not. The event shall be duplicated. (VER034)</para>
      </listitem>
      <listitem>
	<para>BUG033 This text is shown as info (see button with caption '?') of the class AnalogClockProjectManual. There is no version information for this text.
Solution: this is partially true: version information from what is in the image can be found with button "versions". Older versions need to be loaded via Monticello. (VER029)</para>
      </listitem>
      <listitem>
	<para>BUG034 (VER029) Drop an ical-event on the clock while the spiral is not activated. Activate spiral. -> indexing is not correct any more. Debugger points to: Spiral>>rotateEventsAt: in line index:=MySpiralCalendar. (VER034)</para>
      </listitem>
      <listitem>
	<para>BUG035 (VER029) an *.ics-file may contain more than one event. When you drop the corresponding TextMorph on the AnalogClock then you will collect an ICCalendar with two events. But in Spiral>>rotateEventsAt: it will cause an error to retrieve eventShapes at: 2. Probably we are not able to import more than one events. (VER030)</para>
      </listitem>
      <listitem>
	<para>BUG036 (VER029) ---duplicate to B030--- there is a delay of 1 minute between the linux clock shown in the status bar and AnalogClock. (VER034)</para>
      </listitem>
      <listitem>
	<para>BUG036 (VER029) ---duplicate to B030--- there is a delay of 1 minute between the linux clock shown in the status bar and AnalogClock. (VER034)</para>
      </listitem>
      <listitem>
	<para>BUG037 (VER001) ---no bug--- (VER001)</para>
      </listitem>
    </itemizedlist>
<para>---everything resolved above this line---</para>
  <itemizedlist spacing="compact">
      <listitem>
	<para>BUG038 (VER034) drop two ical-events with longer names and beginning at "quarter past" or "quarter to" on the spiraclock. The name of the event which comes next is clipped by the stripe one hour later. It should be seen without clipping. (VERxxx)</para>
      </listitem>
      <listitem>
	<para>BUG039 (VER034) create an AnalogClock, use the menu button of the Halos to create a ConfigPanel. Duplicate the AnalogClock. It does not have a ConfigPanel. But it should.(VERxxx)</para>
      </listitem>
      <listitem>
	<para>BUG040 (VER050) Pharo 1.1, create an AnalogClock, the watchhand polygons seem not to be filled on the thick end of the watchhand. This is different in Squeak 4.1. (VERxxx)</para>
      </listitem>
      <listitem>
	<para>BUG041 (VER052) Steps to reproduce: Squeak 4.1,9957, create an AnalogClock, look at the seconds watchhand. Observed behavior: The duration of the steps is not constant. This creates the impression of something which is stumbling. Expected behavior: Duration of the steps shall be constant. Deviation shall be smaller than 300ms. (VERxxx) </para>
      </listitem>
      <listitem>
	<para>BUG042 (VER054) Steps to reproduce: Squeak 4.2,10966, draw a polygon in order to replace the seconds watchhand, drop it on the AnalogClock, the new shape is used. Now drag the clock to another place. The seconds watchhand is shown at incorrect places. Expected behavior: The seconds watchhand moves with the clock. (VER057)</para>
      </listitem>
      <listitem>
	<para>BUG043 (VER054) Steps to reproduce: Squeak 4.2,10966, Switch on SpiraClock, drag the clock to another place. The spiral is not moved synchronously with the clock but somehow behind it leaving traces. Expected behavior: The spiral moves with the clock. (VER057)</para>
      </listitem>
      <listitem>
	<para>BUG044 (VER057) A test is missing to prove that we accept a PolygonMorph dropped on us and that we change the seconds watchhand accordingly. (VERxxx)</para>
      </listitem>
      <listitem>
	<para>BUG045 (VER057) A test is missing to prove that we can drag and move a clock with spiral on and that the spiral remains synchronized to the clock. (VERxxx)</para>
      </listitem>
      <listitem>
	<para>BUG046 (VER057) A test is missing to prove that we can resize a clock with spiral on and that the spiral remains synchronized to the clock. (VERxxx)</para>
      </listitem>
      <listitem>
	<para>BUG047 (VER062) Add a "second"-button to the ConfigPanel. One advantage of this would be that we are able to watch slowly the movement of a SpiralEvent when we pass from second "59" to second "0". This would help with debugging. (VER063)</para>
      </listitem>
      <listitem>
	<para>BUG048 (VER062) Versions: Squeak 4.2-10966. Steps to reproduce: Set a spiral event in the future. Let the current time run over the whole duration of the event. 1. When approaching the start time this start time is shown incremented by 1 minute. 2. When leaving the end time this end time is shown incremented by 1 minute. Reproducible=yes. (VER063)</para>
      </listitem>
      <listitem>
	<para>BUG049 (VER062) Versions: Squeak 4.2-10966. Steps to reproduce: Show a spiral event. The summary is shown below the figures on the dial. This is unnatural. It should be above in the z-order. Reproducible=yes. (VERxxx)</para>
      </listitem>
      <listitem>
	<para>BUG050 (VER064) The seconds watchhand is under the date. (VER064)</para>
      </listitem>
      <listitem>
	<para>BUG051 (VER064) RQF003 does not work any more.(VER064)</para>
      </listitem>
      <listitem>
	<para>BUG052 (VER064) When the day in date is two digits and when the month in date is september the date field gets longer than the radius of the clock.</para>
      </listitem>
      <listitem>
	<para>BUG053 (VER064) Look at the frame in the Date field. If you display "11 february" the border touches the "y" of february.</para>
      </listitem>
      <listitem>
	<para>BUG054 (VERxxx) .</para>
      </listitem>
    </itemizedlist>
    </section>
<index type="RQF"><title>Index - Functional Requirements</title></index>
<index type="RQN"><title>Index - Non-functional Requirements</title></index>
<index type="RQP"><title>Index - Process and Tools Requirements</title></index>
<index type="VER"><title>Index - Versions</title></index>
<index type="BUG"><title>Index - Bugs</title></index>
</article>!

Object subclass: #AnalogClockInterface
	instanceVariableNames: 'clockDistributor viewController watchhands'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PVpub-AnalogClock'!

!AnalogClockInterface commentStamp: 'pv 11/10/2011 17:34' prior: 0!
An AnalogClockInterface is an interface to the application. It shall be used by unit tests and by configuration panels. The AnalogClockInterface is created by the AnalogClock when its initialization is finished.


Instance Variables
	configPanel:		<ConfigPanel>
	clockDistributor:	<ClockDistributor>
	watchhands:		<WatchHand>

clockDistributor 
	- controls all dynamical (timing) aspects of the clock.
	
viewController 
	- provides control of the clock

watchhands
	- the embedded and moving parts of the clock
	
	
Responsibilities <Collaborating classes>
	I provide an interface used by Tests and ConfigPanel
	<AnalogClock><MorphExtension>
	
!

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 8/20/2011 11:25'!
activeHand
	^ viewController owner world activeHand! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 8/20/2011 11:23'!
center
	^ viewController center! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 8/15/2011 22:28'!
clockDistributor: aClockDistributor 
	clockDistributor := aClockDistributor! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 8/20/2011 11:12'!
configPanel
	^ viewController configPanel ! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 8/20/2011 11:21'!
dateOn
	^ viewController extension valueOfProperty: #dateOn! !

!AnalogClockInterface methodsFor: 'initialize-release' stamp: 'pv 8/20/2011 11:33'!
delete
	^ viewController owner delete! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 8/20/2011 11:04'!
doOneCycleNow
	viewController owner world doOneCycleNow.! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 8/20/2011 08:15'!
extent: aPoint 	
	viewController extent: aPoint! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 8/20/2011 11:26'!
handleDropMorph: dropEvent 
	viewController owner handleDropMorph: dropEvent! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 6/24/2011 23:40'!
hideWatchhand: aSymbol yes: aBoolean
	aBoolean 
		ifTrue: [(watchhands at: aSymbol) hide]
		ifFalse: [(watchhands at: aSymbol) show].! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 8/20/2011 11:13'!
inc: aBoolean 
	clockDistributor changingOffset: clockDistributor changingOffset not.
	clockDistributor inc: aBoolean.
	viewController extension setProperty: #dateAndTimeSource toValue: #osClockWithOffset! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 8/20/2011 11:06'!
initializeConfigPanel 
	viewController initializeConfigPanel ! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 6/24/2011 23:38'!
isHiddenWatchhand: aSymbol
	^ (watchhands at: aSymbol) visible not! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 8/15/2011 22:31'!
jitterOk 
	^ clockDistributor jitterOk! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 8/15/2011 22:27'!
offsetIncrement: aDuration 
	"replaces setIncrementInMinutes:"
	clockDistributor  offsetIncrement: aDuration! !

!AnalogClockInterface methodsFor: 'initialize-release' stamp: 'pv 8/20/2011 11:42'!
removeConfigPanel
	viewController removeConfigPanel! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 8/20/2011 11:12'!
reset
	clockDistributor 
		offset: (Duration
				days: 0
				hours: 0
				minutes: 0
				seconds: 0).
	clockDistributor changingOffset: false.
	viewController extension setProperty: #dateAndTimeSource toValue: #osClock! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 8/20/2011 10:43'!
running
	^ viewController  extension valueOfProperty: #running! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 8/20/2011 10:44'!
running: aBoolean 
	viewController extension setProperty: #running toValue: aBoolean.
	clockDistributor running: aBoolean! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 8/20/2011 10:53'!
spiraClockOn
	^ viewController extension valueOfProperty: #spiraClockOn! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 8/20/2011 08:10'!
spiral
	^ watchhands at: #Spiral! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 8/20/2011 11:23'!
tickmarksOn
	^viewController extension valueOfProperty: #tickmarksOn! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 8/20/2011 11:53'!
toggleConfigPanel
	^ viewController toggleConfigPanel! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 8/20/2011 11:25'!
toggleDate
		"071112pv this method is triggered when someone is using the Halo-menu as specified in addCustomMenuItems:hand:"
	(viewController extension valueOfProperty: #dateOn) 
		ifTrue: [viewController hideDate]
		ifFalse: [viewController showDate].
	viewController extension setProperty: #dateOn toValue: (viewController extension valueOfProperty: #dateOn) not! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 8/20/2011 11:22'!
toggleSpiraClock
		"071112pv this method is triggered when someone is using
	the Halo-menu as specified in addCustomMenuItems:hand:"
	| spiral |
	spiral := viewController watchhands at: #Spiral.
	(viewController extension valueOfProperty: #spiraClockOn)
		ifTrue: [spiral hide]
		ifFalse: [spiral syncMidnightShapeWith: viewController.
			spiral show].
	viewController extension setProperty: #spiraClockOn toValue: (viewController extension valueOfProperty: #spiraClockOn) not! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 8/20/2011 11:21'!
toggleTickmarks
	(viewController extension valueOfProperty: #tickmarksOn) 
		ifTrue: [viewController hideTickmarks]
		ifFalse: [viewController showTickmarks].
	viewController extension setProperty: #tickmarksOn toValue: (viewController extension valueOfProperty: #tickmarksOn) not! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 8/20/2011 08:15'!
viewController: vc 	
	viewController := vc  ! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 8/20/2011 23:54'!
watchhandHour
	^ watchhands at: #Hour! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 8/20/2011 08:05'!
watchhandMin
	^ watchhands at: #Min! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 8/20/2011 08:07'!
watchhandSec
	^ watchhands at: #Sec! !

!AnalogClockInterface methodsFor: 'access' stamp: 'pv 6/25/2011 01:10'!
watchhands: aWatchhands
	watchhands := aWatchhands.! !

Object subclass: #PerformanceEvaluator
	instanceVariableNames: 'secondsDrawMinDeltaMs secondsDrawMaxDeltaMs valueMsArray valueMsIndex leftLimit rightLimit valueMsArrayShare percentageOffLimitsThreshold'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PVpub-AnalogClock'!

!PerformanceEvaluator commentStamp: 'pv 7/9/2011 19:38' prior: 0!
A PerformanceEvaluator is able to judge about the jitter of the seconds watchhand. We measure how this watchhand is really displayed. And we compare this to our expectation to see a movement of the seconds watchhand each second.
The measurement is done during WatchHand>>drawOn:.

Instance Variables
	leftLimit:							<Integer>
	percentageOffLimitsThreshold:		<Float>
	rightLimit:							<Integer>
	secondsDrawMaxDeltaMs:			<Integer>
	secondsDrawMinDeltaMs:			<Integer>
	valueMsArray:						<Array>
	valueMsArrayShare:					<Float>
	valueMsIndex:						<Integer>

leftLimit
	- low limit for time between two drawOn: calls of the seconds watchhand

percentageOffLimitsThreshold
	- what percentage of measurements do we allow to be off limits? Being above this percentage would yield percentageOffLimitsOk to return false.

rightLimit
	- high limit for time between two drawOn: calls of the seconds watchhand

secondsDrawMaxDeltaMs
	- track the maximum of delta time between two drawOn: calls of the seconds watchhand

secondsDrawMinDeltaMs
	- track the minimum of delta time between two drawOn: calls of the seconds watchhand

valueMsArray
	- set array element to 0 if inside the limits, set to 1, if outside

valueMsArrayShare
	- share of array elements being equal or outside the limits

valueMsIndex
	- write index for valueMsArray
	
Responsibilities <Collaborating classes>
	I evaluate jitter
		
!

!PerformanceEvaluator methodsFor: 'measurements' stamp: 'pv 7/8/2011 21:45'!
addDeltaMs: aValueMs
	| valueMsArraySum |
	
	self pvtUpdateMinMax: aValueMs.
	
	"if inside the limits set to 0, if outside set to 1"		
	valueMsArray at: valueMsIndex put: 
		(((aValueMs > leftLimit) & (aValueMs < rightLimit)) ifTrue: [0] ifFalse: [1]).
	valueMsIndex := (valueMsIndex \\ valueMsArray size) + 1.
	valueMsArraySum := valueMsArray inject: 0 into: [:sum :each | sum + each].
	valueMsArrayShare := (valueMsArraySum asFloat / valueMsArray size) * 100.
			
		! !

!PerformanceEvaluator methodsFor: 'initialize-release' stamp: 'pv 1/12/2011 22:18'!
initialize
	secondsDrawMinDeltaMs := 60000.
	secondsDrawMaxDeltaMs := 0.
	
	leftLimit := 0.
	rightLimit := 0.
	"pessimistic assumption: all time intervals are off limits"
	valueMsArray := Array new: 100 withAll: 1.
	valueMsIndex := 1.
! !

!PerformanceEvaluator methodsFor: 'access' stamp: 'pv 1/11/2011 22:41'!
leftLimit: aValueMs
	leftLimit := aValueMs.! !

!PerformanceEvaluator methodsFor: 'initialize-release' stamp: 'pv 6/4/2011 01:00'!
percentageOffLimitsOk
	"Transcript show: valueMsArrayShare asString."
	^valueMsArrayShare <= percentageOffLimitsThreshold ! !

!PerformanceEvaluator methodsFor: 'access' stamp: 'pv 5/27/2011 12:41'!
percentageOffLimitsThreshold: p
	percentageOffLimitsThreshold := p! !

!PerformanceEvaluator methodsFor: 'measurements' stamp: 'pv 7/8/2011 21:45'!
pvtUpdateMinMax: aValueMs
	"set min and max"
	(aValueMs < secondsDrawMinDeltaMs)
		ifTrue: [secondsDrawMinDeltaMs := aValueMs].
	(aValueMs > secondsDrawMaxDeltaMs)
		ifTrue: [secondsDrawMaxDeltaMs := aValueMs]  ! !

!PerformanceEvaluator methodsFor: 'access' stamp: 'pv 1/11/2011 22:41'!
rightLimit: aValueMs
	rightLimit := aValueMs.! !

Object subclass: #PolarCoordinates
	instanceVariableNames: 'r phi'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PVpub-AnalogClock'!

!PolarCoordinates commentStamp: 'pv 7/9/2011 19:39' prior: 0!
A PolarCoordinates is a pair of coordinates consisting of an angle phi and a radius r.

Instance Variables
	phi:		<Number>
	r:		<Number>

phi
	- angle

r
	- radius
	
Responsibilities <Collaborating classes>
	I provide polar coordinates transformation and computing.!

!PolarCoordinates class methodsFor: 'as yet unclassified' stamp: 'pv 6/13/2011 16:28'!
r: aRadius phi: aRadian
	^ self new
		r:  aRadius;
		phi: aRadian;
		yourself! !

!PolarCoordinates methodsFor: 'accessing' stamp: 'pv 5/29/2011 17:01'!
phi 
	^ phi! !

!PolarCoordinates methodsFor: 'accessing' stamp: 'pv 5/29/2011 16:49'!
phi: aNumber
	phi := aNumber! !

!PolarCoordinates methodsFor: 'accessing' stamp: 'pv 5/29/2011 17:01'!
r
	^ r! !

!PolarCoordinates methodsFor: 'accessing' stamp: 'pv 5/29/2011 16:49'!
r: aNumber
	r := aNumber! !

Object subclass: #SpiralCalendar
	instanceVariableNames: 'mySpiral'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PVpub-AnalogClock'!

!SpiralCalendar commentStamp: 'pv 7/9/2011 19:41' prior: 0!
A SpiralCalendar is a representation of several events which you have planned. It gets its data from a personal information management (PIM)-application (say Linux/KDE/KOrganizer). It shows these data inside the dial area of our AnalogClock. A single event is shown as a segment of a spiral. The graphical representation of the events is an OrderedCollection of SpiralEvent(s). 

It is static as far as angles of its radial components will remain unchanged.
It is dynamic since its radial coordinates will move with the Spiral

The SpiralCalendar is responsible of rotating its SpiralEvents when receiving rotationIsDueAt: rotateAt: or syncMidnightShapeWith:
The SpiralCalendar is responsible for Spiral when it comes to switch on/off the whole spiral functionality.

Instance Variables
	eventShapes:		<SpiralEvent>
	mySpiral:			<Spiral>

eventShapes
	- shape corresponding to an event and fitting into the spiral
	
mySpiral
	- spiral with all geometrical information
	
Responsibilities <Collaborating classes>
	I store and transform calendar evetns
	<Spiral>
		
!

!SpiralCalendar class methodsFor: 'as yet unclassified' stamp: 'pv 3/17/2013 20:42'!
newImportedEventFrom: aString
	| calendar newCalendarEvents oldCalendarEvents fileContent |
	calendar := ICCalendar name: 'test'.
	oldCalendarEvents := OrderedCollection newFrom: calendar events.
	"110527 import adds new event to the existing event collection"
	fileContent := (SystemVersion current version beginsWith: 'Squeak') 
		ifTrue: [aString withoutTrailingBlanks withInternetLineEndings]
		ifFalse: [aString trimRight withInternetLineEndings].
	ICImporter new
		calendar: calendar;
		fileContent: fileContent;
		import.
	newCalendarEvents := OrderedCollection newFrom: calendar events.
	"set difference"
	^ newCalendarEvents removeAllSuchThat: [ :elem | oldCalendarEvents includes: elem]! !

!SpiralCalendar methodsFor: 'import' stamp: 'pv 2/4/2012 19:31'!
addEvent: aString 
	"Import one single event. 
	(design question: What about multiple events in one step ?)"
	"self flag: #RQF009; flag: #RQF012; flag: #RQF016."
	| spiralEvent newEvents |
	newEvents := self class newImportedEventFrom: aString.
	newEvents
		do: [:each | 
			spiralEvent := SpiralEvent new.
			"a PolygonMorph fitting into the spiral and a summary
			label for it"
			spiralEvent startTime: each start;
				 endTime: each end;
				 setSummaryContents: each summary.
			"a label with explaining text from ical-SUMMARY"
			mySpiral addMorph: spiralEvent;
				 justDropped: true]! !

!SpiralCalendar methodsFor: 'accessing' stamp: 'pv 8/31/2010 20:04'!
mySpiralIs: aSpiral
	mySpiral := aSpiral! !

CircleMorph subclass: #ClockDistributor
	instanceVariableNames: 'goForwardInTime changingOffset offsetIncrement offset dateAndTimeNowFrozen'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PVpub-AnalogClock'!

!ClockDistributor commentStamp: 'pv 11/10/2011 18:48' prior: 0!
A ClockDistributor is the source of dynamics of the clock. It hosts the one and only step method. This step method acts on all dynamic submorphs of AnalogClock. For instance the WatchHand and the DateMorph.
On the other hand a ClockDistributor does not have anything to draw. Its surface is zero.

Instance Variables
	changingOffset:			<Boolean>
	dateAndTimeNowFrozen 	<DateAndTime>
	goForwardInTime:			<Boolean>
	offset:						<Object>
	offsetIncrement: 			<Duration>

changingOffset
	- we add an offset to the frozen time and this offset changes at each step
	
dateAndTimeNowFrozen
	- the value of DateAndTime>>now at the moment when the user asks for an offset

goForwardInTime
	- do we add or subtract the offset?
	
offset
	- the duration to add to the frozen time

offsetIncrement
	- the duratiion to add to the offset when we increment or decrement
	
	
Responsibilities <Collaborating classes>
	I control moving visible elements of the clock
	<DateMorph><Spiral><WatchHand><Dial>
	
!

!ClockDistributor methodsFor: 'clock-offset' stamp: 'pv 1/9/2011 01:02'!
changingOffset
	^changingOffset! !

!ClockDistributor methodsFor: 'clock-offset' stamp: 'pv 1/9/2011 01:02'!
changingOffset: aBoolean
	changingOffset := aBoolean! !

!ClockDistributor methodsFor: 'clock-offset' stamp: 'pv 1/9/2011 01:04'!
inc: aBoolean
	goForwardInTime := aBoolean.! !

!ClockDistributor methodsFor: 'parts bin' stamp: 'pv 7/24/2011 20:07'!
initialize
	"self flag: #RQF005.	"
	super initialize.
	self extent: 0 @ 0.
	offset := Duration
				days: 0
				hours: 0
				minutes: 0
				seconds: 0.
	offsetIncrement := Duration
				days: 0
				hours: 0
				minutes: 1
				seconds: 0.
	goForwardInTime := true.
	changingOffset := false! !

!ClockDistributor methodsFor: 'accessing' stamp: 'pv 2/13/2011 23:31'!
jitterOk
	^ (self owner watchhandSec jitterOk)
! !

!ClockDistributor methodsFor: 'clock-offset' stamp: 'pv 7/24/2011 20:07'!
offset: aDuration 
	offset := aDuration! !

!ClockDistributor methodsFor: 'clock-offset' stamp: 'pv 7/24/2011 20:02'!
offsetIncrement: aDuration 
	"replacement for incrementInMinutes:"
	offsetIncrement := aDuration! !

!ClockDistributor methodsFor: 'stepping and presenter' stamp: 'pv 11/15/2011 23:37'!
pvtCurrentTimeWithOffset
	| dateAndTimeNow |
	dateAndTimeNow := DateAndTime now.
	self owner configuredWithOffset
		ifTrue: [self owner newlyConfiguredWithOffset
				ifTrue: [dateAndTimeNowFrozen := dateAndTimeNow].
			self pvtSetOffset.
			dateAndTimeNow := dateAndTimeNowFrozen + offset].
	^ dateAndTimeNow! !

!ClockDistributor methodsFor: 'stepping and presenter' stamp: 'pv 11/15/2011 22:53'!
pvtSetOffset
	changingOffset
		ifTrue: [offset := goForwardInTime
						ifTrue: [offset + offsetIncrement]
						ifFalse: [offset - offsetIncrement]].! !

!ClockDistributor methodsFor: 'drawing' stamp: 'pv 6/13/2011 16:28'!
rotateLazyAt: aDateAndTime
	^ self! !

!ClockDistributor methodsFor: 'accessing' stamp: 'pv 8/15/2011 23:07'!
running
	^ self owner running! !

!ClockDistributor methodsFor: 'accessing' stamp: 'pv 8/15/2011 23:07'!
running: aBoolean
	^ self owner running: aBoolean.! !

!ClockDistributor methodsFor: 'stepping and presenter' stamp: 'pv 3/14/2013 23:21'!
step
	| dateAndTimeNow |
	"self
		flag: #RQF004;
		flag: #RQF021."
	"ClockDistributor is the only responsible to prepare all submorphs of AnalogClock for the next call of drawOn:"
	"This is done to provide one and the same time to all instances of WatchHand."
	"AnalogClock would not be able to do the same since step is only accepted for submorphs. ClockDistributor is a submorph of AnalogClock."
	self running ifFalse: [ ^ self ].
	dateAndTimeNow := self pvtCurrentTimeWithOffset.
	self owner rotateLazyAt: dateAndTimeNow! !

!ClockDistributor methodsFor: 'testing' stamp: 'pv 1/12/2011 23:01'!
stepTime
	"071022pv make sure that we do not 'forget' a second - this is possible with stepTime = 1000.
	Make sure that we do not have unequeal periods for one second - this is possible with stepTime between 200 and 1000."
	^ 43
! !

CircleMorph subclass: #Tickmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PVpub-AnalogClock'!

!Tickmark commentStamp: 'pv 7/9/2011 19:42' prior: 0!
A TickmarkMorph represents the tickmarks on the dial.

Instance Variables

Responsibilities <Collaborating classes>
	I represent tickmarks!

!Tickmark methodsFor: 'parts bin' stamp: 'pv 5/3/2008 16:32'!
initialize
	super initialize.
	self extent: (3 @ 3);
		color: Color blue.! !

!Tickmark methodsFor: 'drawing' stamp: 'pv 8/13/2011 23:40'!
rotateLazyAt: dateAndTime 
	^ self! !

RectangleMorph subclass: #ConfigPanel
	instanceVariableNames: 'startButton hourHideButton minHideButton secHideButton myAnalogClockRC'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PVpub-AnalogClock'!

!ConfigPanel commentStamp: 'pv 11/10/2011 18:48' prior: 0!
A ConfigPanel is a RectangleMorph with several buttons embedded. This is to let a user configure and control the AnalogClock.
It also shows different states of the AnalogClock: refer to AnalogClock.


Instance Variables
	hourHideButton:		<SetButton>
	minHideButton:			<SetButton>
	myAnalogClockRC:		<AnalogClockAccess>
	secHideButton:			<SetButton>
	startButton:				<SetButton>

hourHideButton
	- hide hour watchhand

minHideButton
	- hide minutes watchhand

myAnalogClockRC
	- provides access to analog clock

secHideButton
	- hide seconds watchhand

startButton
	- start/stop clock
	
Responsibilities <Collaborating classes>
	I provide a panel for the user. He can use it for manual tests.
	<AnalogClockInterface>
	
!

!ConfigPanel methodsFor: 'accessing' stamp: 'pv 7/8/2011 21:37'!
connectTo: anAnalogRemoteControl	
	myAnalogClockRC := anAnalogRemoteControl.
	"initialization of buttons depends on access to state of clock"
	self pvtInitializeButtons	! !

!ConfigPanel methodsFor: 'menu' stamp: 'pv 6/24/2011 23:25'!
dec
	myAnalogClockRC inc: false.! !

!ConfigPanel methodsFor: 'menu' stamp: 'pv 6/24/2011 23:28'!
inc
	myAnalogClockRC inc: true.
! !

!ConfigPanel methodsFor: 'menu' stamp: 'pv 7/24/2011 19:57'!
incrementIsHour
	myAnalogClockRC offsetIncrement: (Duration days: 0 hours: 1 minutes: 0 seconds: 0)! !

!ConfigPanel methodsFor: 'menu' stamp: 'pv 7/24/2011 19:58'!
incrementIsMin
	myAnalogClockRC
		offsetIncrement: (Duration
				days: 0
				hours: 0
				minutes: 1
				seconds: 0)! !

!ConfigPanel methodsFor: 'menu' stamp: 'pv 7/24/2011 19:58'!
incrementIsSec
	myAnalogClockRC
		offsetIncrement: (Duration
				days: 0
				hours: 0
				minutes: 0
				seconds: 1)! !

!ConfigPanel methodsFor: 'initialization' stamp: 'pv 6/4/2011 00:18'!
initialize
	| |
	"self flag: #RQF021."
	super initialize.
	self
		extent: 150 @ 200;
		name: 'AnalogClock-Configurator';
		layoutPolicy: TableLayout new;
		listDirection: #topToBottom;
		listCentering: #topLeft;
		wrapCentering: #center.	"lay out contents as a table"	"how we want to place the contents"	"start list at the top"	"each item is in the center"	
	self
		cellInset: 2 @ 5;
		hResizing: #shrinkWrap;
		layoutInset: 4 @ 8;
		vResizing: #shrinkWrap.	"controls distance between content elements"	"try it and see!!"	"that was a bit too cramped"	"Now we are done"
	"Now that submorphs are placed we know the real extent of configPanel. Find a good position."		
	"c... openInWorld"
	"changingOffset := false."	"show the current time without an offset"
! !

!ConfigPanel methodsFor: 'initialization' stamp: 'pv 7/8/2011 21:39'!
pvtInitializeButtons
	"bottom up"
	| |
	.
	self pvtInitializeWatchhandButtons.
	self pvtInitializeSimpleButtons.			.
	startButton := self 
		pvtInitializeConfigPanelButtonLabeled: (myAnalogClockRC running 
			ifTrue: ['stop']
			ifFalse: ['start']) 
		Selector: #toggleStartStop.	
! !

!ConfigPanel methodsFor: 'initialization' stamp: 'pv 2/26/2013 10:48'!
pvtInitializeConfigPanelButtonLabeled: aString Selector: aSymbol
	| aButton |
	"UpdateSimpleButtonMorph not known in Pharo"
	aButton := SimpleButtonMorph new.
	aButton
		target: self;
		label: aString;
		actionSelector: aSymbol;
		extent: 70@18;
		hResizing: #spaceFill.	
	self addMorph: aButton.
	^ aButton! !

!ConfigPanel methodsFor: 'initialization' stamp: 'pv 7/24/2011 19:33'!
pvtInitializeSimpleButtons
	"bottom up"
	self pvtInitializeConfigPanelButtonLabeled: 'inc' Selector: #inc;
		 pvtInitializeConfigPanelButtonLabeled: 'dec' Selector: #dec;
		 pvtInitializeConfigPanelButtonLabeled: 'sec' Selector: #incrementIsSec;
		 pvtInitializeConfigPanelButtonLabeled: 'min' Selector: #incrementIsMin;
		 pvtInitializeConfigPanelButtonLabeled: 'hour' Selector: #incrementIsHour;
		 pvtInitializeConfigPanelButtonLabeled: 'reset' Selector: #reset! !

!ConfigPanel methodsFor: 'initialization' stamp: 'pv 7/8/2011 21:39'!
pvtInitializeWatchhandButtons
	secHideButton := self 
		pvtInitializeConfigPanelButtonLabeled: ((myAnalogClockRC isHiddenWatchhand: #Sec)   
			ifTrue: ['show second']
			ifFalse: ['hide second']) 
		Selector: #pvtToggleHideSec.				
	minHideButton := self 
		pvtInitializeConfigPanelButtonLabeled: ((myAnalogClockRC isHiddenWatchhand: #Min)   
			ifTrue: ['show minute']
			ifFalse: ['hide minute']) 
		Selector: #pvtToggleHideMin.			
	hourHideButton := self 
		pvtInitializeConfigPanelButtonLabeled: ((myAnalogClockRC isHiddenWatchhand: #Hour)   
			ifTrue: ['show hour']
			ifFalse: ['hide hour']) 
		Selector: #pvtToggleHideHour.	! !

!ConfigPanel methodsFor: 'menu' stamp: 'pv 7/8/2011 21:34'!
pvtToggleHideHour
	myAnalogClockRC hideWatchhand: #Hour yes: (myAnalogClockRC isHiddenWatchhand: #Hour) not.
	hourHideButton label: 
		((myAnalogClockRC isHiddenWatchhand: #Hour) 
			ifTrue: ['show hour']
			ifFalse: ['hide hour'])! !

!ConfigPanel methodsFor: 'menu' stamp: 'pv 7/8/2011 21:34'!
pvtToggleHideMin
	myAnalogClockRC hideWatchhand: #Min yes: (myAnalogClockRC isHiddenWatchhand: #Min) not.
	minHideButton label: 
		((myAnalogClockRC isHiddenWatchhand: #Min) 
			ifTrue: ['show minute']
			ifFalse: ['hide minute'])! !

!ConfigPanel methodsFor: 'menu' stamp: 'pv 7/8/2011 21:34'!
pvtToggleHideSec
	myAnalogClockRC hideWatchhand: #Sec yes: (myAnalogClockRC isHiddenWatchhand: #Sec) not.
	secHideButton label: 
		((myAnalogClockRC isHiddenWatchhand: #Sec) 
			ifTrue: ['show second']
			ifFalse: ['hide second'])! !

!ConfigPanel methodsFor: 'initialization' stamp: 'pv 8/20/2011 12:12'!
remove
	myAnalogClockRC := nil.! !

!ConfigPanel methodsFor: 'menu' stamp: 'pv 6/24/2011 23:31'!
reset
	myAnalogClockRC reset! !

!ConfigPanel methodsFor: 'menu' stamp: 'pv 6/24/2011 23:35'!
toggleStartStop
	myAnalogClockRC running: myAnalogClockRC running not.
	startButton	label: 
		(myAnalogClockRC running 
			ifTrue: ['stop']
			ifFalse: ['start'])! !
Spiral initialize!
DateText initialize!
